# (C) http://spcms.ru
use locale;
use strict;

use vars
  qw/%g_period_to_str %g_period_to_name %g_period_id_to_name $gteacher_show_all/;

our %prev_period = (
    t3 => 't2',
    t2 => 't1',
    q4 => 'q3',
    q3 => 'q2',
    q2 => 'q1',
    y  => 't1',
    t1 => 't',
    q1 => 't',
    t  => 't',
    e  => 'e',
);

sub report_form() {
    my $periods_mode =
      sqlff 'SELECT F_INT FROM SP_SETTINGS WHERE NAME = \'periods_mode\'';
    my $periods;
    if ($periods_mode) {
        for ( 1 .. 4 ) {
            $periods .=
qq[<input type=radio id="idq$_" name="p" value="q$_"><label for="idq$_">$_ <br></label>];
        }
    }
    else {
        for ( 1 .. 3 ) {
            $periods .=
qq[<input type=radio id="idt$_" name="p" value="t$_"><label for="idt$_">$_ <br></label>];
        }
    }

    my $filter_class;
    if ( $ggid == 2 ) {
        $filter_class =
          $gteacher_show_all ? filter_class(0) : filter_class_teacher_binded(0);
    }
    else {
        $filter_class = filter_class(0);
    }

    my $halfs =
      ( sqlff "select f_int from sp_settings where name = 'enable_half_year'" )
      ? q[<input type=radio id="idh1" name="p" value="h1"><label for="idh1">1 <br></label>
<input type=radio id="idh2" name="p" value="h2"><label for="idh2">2 <br></label>]
      : '';

    my %replace = (
        menu    => generate_menu(),
        class   => $filter_class,
        periods => $periods,
        halfs   => $halfs,
        footers => generate_footers(),
    );
    tpl( './tpl/report/class_form.html', \%replace );
}

sub report {
    my ( $for_parallel, $for_class, $for_period, $out_ref );
    $for_class = shift;
    $for_parallel = ($for_class) ? 1 : 0;
    if ($for_parallel) {
        ( $for_period, $out_ref ) = ( shift, shift );
    }

    my @a;
    my $q;

    my $c = ($for_parallel) ? $for_class  : param('c');
    my $p = ($for_parallel) ? $for_period : param('p');
    if   ( defined $c ) { sec_gt0 $c; }
    else                { stop "   ."; }

    my $parallel = sqlff "select paralel from sys_matrix
    where sys_matrix.view_class = 7 and class = $c and
    exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)";
    unless ($parallel) {
        if ($for_parallel) { return undef; }
        stop '   .';
    }
    if ( $p == 10 or $p == 11 ) {
        unless ( grep /^$parallel$/,
            split /,/,
            sqlff "select f_char from sp_settings where name = 'half_year'" )
        {
            stop '     .';
        }
    }

    my ( $year_total, undef ) = year_academic();

    my @year_set = sqlf
      'select time_start, time_end from sp_periods where link = 0 and id = 0';
    $_ = todate($_) for (@year_set);

    $q = $gdb->prepare(
        qq[select sys_entry.doc_class, sys_entry.entry_date
from sys_entry where
sys_entry.entry_date between '$year_set[0]' and '$year_set[1]' and
(sys_entry.doc_class = 7 and
 exists(select 1 from sys_matrix where
        sys_entry.credit_node = node_id and
        view_class = 7 and class = $c)
   or
 sys_entry.doc_class = 8 and
 exists(select 1 from sys_matrix where
        sys_entry.credit_node = node_id and
        view_class = 7 and class = $c))], { ib_timestampformat => "%d.%m.%Y" }
    ) or db_error;
    $q->execute;
    my ( @class_actions, @tmp );
    while ( @tmp = $q->fetchrow_array ) { push @class_actions, [@tmp]; }
    for (@class_actions) {
        next if ( totime( $_->[1] ) > totime($gdate) );

        my $prev_day = todate( day_sub( totime( $_->[1] ) ) );
        my $ext_text =
"<br><br><a href=\"?act=state_date_set&d=$prev_day\"></a>    $prev_day.";
        my $class_text =
          ($for_parallel) ? '  ' : '  ';
        if ( $_->[0] == 7 ) {
            msg "",
" $gdate. $class_text     .<br>"
              . "  $_->[1]."
              . $ext_text;
        }
        if ( $_->[0] == 8 ) {
            msg "",
                " $gdate. $class_text  .<br>"
              . "  $_->[1]."
              . $ext_text;
        }
    }

    my $resolved_class;
    unless ($for_parallel) {
        $resolved_class = sqlff "select
      paralel || (select name from class where class_id = cast(sys_matrix.class as integer))
    from
      class, sys_matrix
    where
      class_id = $c
      and sys_matrix.class = class_id
      and sys_matrix.view_class = 7
      and exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)";
        $resolved_class =~ s/^(\d+\".+\").+/$1/;
    }

    my $resolved_period;
    unless ($for_parallel) {
        my $period_valid = 0;
        for ( keys %g_period_to_name ) {
            if ( $p eq $_ ) {
                $resolved_period = $g_period_to_name{$_};
                $period_valid    = 1;
            }
        }
        if ( !$period_valid ) { stop "   ." }
    }

    my ( %subgrp_hash, %subject_hash, %subgrp_to_delete, %subgrp_to_subj,
        @subgrp );
    sqlh \%subgrp_hash,
      "select id,trim(name) from sp_class_subgrp_ref where c = $c";

    sqlh \%subject_hash, 'select lesson_id, trim(name) from lesson';

    sqlh \%subgrp_to_delete,
      "select distinct s,1 from sp_class_subgrp_ref where c = $c";
    sqla \@subgrp, "select s,id from sp_class_subgrp_ref where c = $c";

    my %pupil_count_in_subgrp;
    sqlh \%pupil_count_in_subgrp, "select id, (
select count(1) from sp_class_subgrp
where sp_class_subgrp_ref.id = sp_class_subgrp.id
) from sp_class_subgrp_ref where c = $c";

    my %subject_hash_withot_subgrp = %subject_hash;
    for ( keys %subgrp_to_delete ) {
        delete $subject_hash_withot_subgrp{$_} if exists $subject_hash{$_};
    }

    my %sou_hash;
    $q = sqlp "select s, g, f from sp_req where c = $c and y = $year_total";
    while ( @a = $q->fetchrow_array ) { $sou_hash{"$a[0]g$a[1]"} = $a[2]; }

    my @subjects_with_subgroups;

    for ( keys %subject_hash_withot_subgrp ) {
        push @subjects_with_subgroups, [ $_, undef, $sou_hash{ $_ . 'g' } ];
    }

    for (@subgrp) {
        push @subjects_with_subgroups,
          [ $_->[0], $_->[1], $sou_hash{ $_->[0] . 'g' . $_->[1] } ];
    }

    my %binded_subjects;
    sqlh \%binded_subjects,
      qq[select distinct cast(lesson as integer), 1 from sys_matrix
where view_class = 9001
and class = $c
and exists ( select 1 from get_node_saldo_by_unit ( sys_matrix.node_id, '$gdate', '', 0 )
where saldo_sum <> 0)];
    if (%binded_subjects) {
        my @new_subjects_with_subgroups = ();
        for (@subjects_with_subgroups) {
            push @new_subjects_with_subgroups, $_
              if $binded_subjects{ $_->[0] };
        }
        @subjects_with_subgroups = @new_subjects_with_subgroups;
    }

    @subjects_with_subgroups = sort {
             $subject_hash{ $a->[0] } cmp $subject_hash{ $b->[0] }
          || $subgrp_hash{ $a->[1] } cmp $subgrp_hash{ $b->[1] }
    } @subjects_with_subgroups;

    if ( param 'hide_empty_subjects' ) {
        my %subjects_with_scores;
        sqlh \%subjects_with_scores, "select lesson.lesson_id, 1 from lesson
where exists
(
select 1 from sp_scores_totals
join sys_matrix on uid = vw_pupil
  where sys_matrix.view_class = 1
    and sys_matrix.class = $c
    and sp_scores_totals.scoreyear = $year_total
    and sp_scores_totals.$p is not null
    and sp_scores_totals.subj = lesson.lesson_id
    and exists
    (
       select 1 from get_node_saldo ( sys_matrix.node_id, '$gdate', 0)
       where saldo_sum <> 0
    )
)
or exists
(
select 1 from sp_marks
join sys_matrix on uid = vw_pupil
  where sys_matrix.view_class = 1
    and sys_matrix.class = $c
    and sp_marks.y = $year_total
    and sp_marks.p = $g_period_to_str{$p}
    and sp_marks.s = lesson.lesson_id
    and sp_marks.t in (1,2)
    and exists
    (
       select 1 from get_node_saldo ( sys_matrix.node_id, '$gdate', 0)
       where saldo_sum <> 0
    )
)";

        my @new_subjects_with_subgroups = ();

        for (@subjects_with_subgroups) {
            push @new_subjects_with_subgroups, $_
              if $subjects_with_scores{ $_->[0] };
        }

        @subjects_with_subgroups = @new_subjects_with_subgroups;
    }

    my $subjs_html;
    unless ($for_parallel) {
        for (@subjects_with_subgroups) {
            DrawVerticalText( $$_[0], $subject_hash{ $$_[0] }, 0 )
              unless -e "$gconf{'sppath'}/tmp/img_subj/$$_[0].png";

            if ( $$_[1] ) {
                DrawVerticalText( $$_[1], $subgrp_hash{ $$_[1] }, 1 )
                  unless -e "$gconf{'sppath'}/tmp/img_subgrp/$$_[1].png";
            }

            $subjs_html .=
qq[<td class=rep_col><nobr><img src="$gconf{'sproot'}/tmp/img_subj/$$_[0].png">];
            $subjs_html .=
qq[<img style="margin-left:2px;margin-bottom:10px" src="$gconf{'sproot'}/tmp/img_subgrp/$$_[1].png">]
              if $$_[1];
            $subjs_html .= qq[</nobr></td>];
        }
    }

    $q = sqlp(
        "select last_name, first_name, middle_name, human_id
from
  sys_matrix, human
where
  sys_matrix.class = $c and
  human_id = cast(vw_pupil as integer) and
  sys_matrix.view_class = 1 and
  exists(select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)
order by last_name"
    );

    my @uids;
    my %names;
    while ( @a = $q->fetchrow_array() ) {
        $a[1] = substr( trim( $a[1] ), 0, 1 ) . '.';
        $a[2] = substr( trim( $a[2] ), 0, 1 ) . '.';

        push @uids, $a[3];

        $names{ $a[3] } = "$a[0] $a[1] $a[2]";
    }

    if ( param('sort') eq 'avg' ) {
        my @a;
        my %avgsort;
        $q = sqlp "select uid, avg(cast($p as float))
from sp_scores_totals join sys_matrix on uid = vw_pupil
where sys_matrix.view_class = 1
  and sys_matrix.class = $c
  and sp_scores_totals.scoreyear = $year_total
  and exists
  (
    select 1 from get_node_saldo ( sys_matrix.node_id, '$gdate', 0)
    where saldo_sum <> 0
  )
group by uid";
        while ( @a = $q->fetchrow_array() ) {
            $a[1] = sprintf "%.3f", $a[1];
            $avgsort{"$a[1]u$a[0]"} = $a[0];
        }
        $q->finish;
        my @uids_new;
        push @uids_new, $avgsort{$_} for ( reverse sort keys %avgsort );
        my @uids_old;
        push @uids_old, @uids;
        undef @uids;
        push @uids, @uids_new;
        my @lost;
        my %hash;
        @hash{@uids} = ();
        for (@uids_old) { push( @lost, $_ ) unless exists $hash{$_} }
        push @uids, @lost if @lost;
    }

    my %scores;
    my %marks;
    my %avg_scores;
    my %avg_count;
    my $scores_uid;
    my $scores_uid_count;
    my $avg_score_uid;
     my %scores5;
    my %scores4;
    my %scores3;
    my %scores2;
    my %scoresNA;
    my ($scores5_uid_count);
    my ($scores4_uid_count);
    my ($scores3_uid_count);
    my ($scores2_uid_count);
    my $scoresNA_uid_count;
    my %freed_pupils;

    my ( $all5, $all45, $one3, $one4, $has2, $na, $has2na, $has3s );
    $all5 = $all45 = $one3 = $one4 = $has2 = $na = $has2na = $has3s = 0;

    my $t;
    my $name_color;
    my $uid_scores;
    my $number = 1;
    my $uid_grp;

    my @a2;
    my $q2;

    my %uid_grp;
    sqlh \%uid_grp,
      "select uid||'s'||s, id from sp_class_subgrp, sp_class_subgrp_ref
where
sp_class_subgrp_ref.id = sp_class_subgrp.id
and c = $c";

    foreach my $uid (@uids) {
        $gdb->{"AutoCommit"} = 0;
        undef %scores;
        undef %marks;

        $q = sqlp "select subj, $p from sp_scores_totals
where uid = $uid and scoreyear = $year_total";
        while ( @a = $q->fetchrow_array() ) {
            $uid_grp = $uid_grp{"\Q$uid\Es\Q$a[0]\E"};

            $scores{"$a[0]g$uid_grp"} = $a[1];
        }
        $q->finish;

        $q = sqlp "select s, t from sp_marks
where y = $year_total and p = $g_period_to_str{$p} and uid = $uid";
        while ( @a2 = $q->fetchrow_array() ) {
            $uid_grp =
              sqlff "select id from sp_class_subgrp, sp_class_subgrp_ref
where
sp_class_subgrp_ref.id = sp_class_subgrp.id
and uid = $uid
and c = $c
and s = $a2[0]";
            $marks{"$a2[0]g$uid_grp"} = $a2[1];
        }
        $q->finish;

        $gdb->{"AutoCommit"} = 1;

        $scores_uid_count    = $scores_uid        = $scores5_uid_count =
          $scores4_uid_count = $scores3_uid_count = $scores2_uid_count =
          $scoresNA_uid_count = 0;

        foreach my $sg (@subjects_with_subgroups) {
            $scores5{"$sg->[0]g$sg->[1]"}++
              if $scores{"$sg->[0]g$sg->[1]"} == 5;
            $scores4{"$sg->[0]g$sg->[1]"}++
              if $scores{"$sg->[0]g$sg->[1]"} == 4;
            $scores3{"$sg->[0]g$sg->[1]"}++
              if $scores{"$sg->[0]g$sg->[1]"} == 3;
            $scores2{"$sg->[0]g$sg->[1]"}++
              if $scores{"$sg->[0]g$sg->[1]"} == 2;
            $scores5_uid_count++ if $scores{"$sg->[0]g$sg->[1]"} == 5;
            $scores4_uid_count++ if $scores{"$sg->[0]g$sg->[1]"} == 4;
            $scores3_uid_count++ if $scores{"$sg->[0]g$sg->[1]"} == 3;
            $scores2_uid_count++ if $scores{"$sg->[0]g$sg->[1]"} == 2;

            if ( $marks{"$sg->[0]g$sg->[1]"} == 1 ) {
                $scoresNA_uid_count++;
                $scoresNA{"$sg->[0]g$sg->[1]"}++;
            }

            if ($for_parallel) {
                if ( $scores{"$sg->[0]g$sg->[1]"} ) {
                    $avg_scores{"$sg->[0]g$sg->[1]"} +=
                      $scores{"$sg->[0]g$sg->[1]"};
                    $avg_count{"$sg->[0]g$sg->[1]"}++;
                    $scores_uid += $scores{"$sg->[0]g$sg->[1]"};
                    $scores_uid_count++;
                }
                elsif ( $marks{"$sg->[0]g$sg->[1]"} == 1 ) {
                    $avg_count{"$sg->[0]g$sg->[1]"}++;
                }
                elsif ( $marks{"$sg->[0]g$sg->[1]"} == 2 ) {
                    $freed_pupils{$uid}++;
                }
            }
            else {
                $uid_scores .= '<td>';

                if ( defined( $scores{"$sg->[0]g$sg->[1]"} ) ) {
                    $uid_scores .= $scores{"$sg->[0]g$sg->[1]"};
                    $avg_scores{"$sg->[0]g$sg->[1]"} +=
                      $scores{"$sg->[0]g$sg->[1]"};
                    $avg_count{"$sg->[0]g$sg->[1]"}++;
                    $scores_uid += $scores{"$sg->[0]g$sg->[1]"};
                    $scores_uid_count++;
                }
                elsif ( $marks{"$sg->[0]g$sg->[1]"} == 1 ) {
                    $uid_scores .= "<span class=tot_na>/</span>";
                    $avg_count{"$sg->[0]g$sg->[1]"}++;
                }
                elsif ( $marks{"$sg->[0]g$sg->[1]"} == 2 ) {
                    $uid_scores .= "<span class=tot_freed></span>";
                    $freed_pupils{$uid}++;
                }
                else {
                    $uid_scores .= '&nbsp;';
                }

                if ( $marks{"$sg->[0]g$sg->[1]"} == 3 ) {
                    $uid_scores .= "<br><span class=tot_home></span>";
                }
                $uid_scores .= '</td>';
                $avg_score_uid =
                  $scores_uid_count
                  ? sprintf( "%.3f", $scores_uid / $scores_uid_count )
                  : '&nbsp;';
            }
        }

        unless ($for_parallel) {
            $name_color = '000';
            if (    $scores_uid_count
                and ( $scores_uid_count == $scores5_uid_count )
                and !$scoresNA_uid_count )
            {
                $name_color = '008729';
            }
            elsif ( $scores_uid_count
                and ( $scores_uid_count - $scores5_uid_count == 1 )
                and $scores4_uid_count == 1
                and !$scoresNA_uid_count )
            {
                $name_color = '749600';
            }
            elsif (
                $scores_uid_count
                and ( $scores_uid_count ==
                    ( $scores5_uid_count + $scores4_uid_count ) )
                and !$scoresNA_uid_count
              )
            {
                $name_color = '2364B5';
            }
            elsif ( $scores3_uid_count == 1
                and !$scores2_uid_count
                and !$scoresNA_uid_count )
            {
                $name_color = '8A4A3D';
            }
            elsif ( $scores2_uid_count and !$scoresNA_uid_count ) {
                $name_color = 'F00';
            }
            elsif ( $scores2_uid_count and $scoresNA_uid_count ) {
                $name_color = 'FF6B00';
            }

            $t .=
'<tr align=center OnMouseOver="m_over(this);" OnMouseOut="m_out(this);">';
            $t .=
"<td align=left nowrap style='color: #$name_color'>$number. $names{$uid}</td>";
            $t .= $uid_scores;
            $uid_scores = '';
            $number++;
        }

        if (    $scores_uid_count
            and ( $scores_uid_count == $scores5_uid_count )
            and !$scoresNA_uid_count )
        {
            $all5++;
        }
        elsif ( $scores_uid_count
            and ( $scores_uid_count - $scores5_uid_count == 1 )
            and $scores4_uid_count == 1
            and !$scoresNA_uid_count )
        {
            $one4++;
        }
        elsif (
            $scores_uid_count
            and
            ( $scores_uid_count == ( $scores5_uid_count + $scores4_uid_count ) )
            and !$scoresNA_uid_count
          )
        {
            $all45++;
        }
        elsif ( $scores3_uid_count == 1
            and !$scores2_uid_count
            and !$scoresNA_uid_count )
        {
            $one3++;
        }
        elsif ( $scores3_uid_count > 1
            and !$scores2_uid_count
            and !$scoresNA_uid_count )
        {
            $has3s++;
        }
        elsif ( $scores2_uid_count and !$scoresNA_uid_count ) { $has2++ }
        elsif ( $scores2_uid_count and $scoresNA_uid_count )  { $has2na++ }

        $na++ if ( $scoresNA_uid_count and !$scores2_uid_count );

        $t .= "<td>$avg_score_uid</td>";

        unless ($for_parallel) {
            $scores5_uid_count  ||= '&nbsp;';
            $scores4_uid_count  ||= '&nbsp;';
            $scores3_uid_count  ||= '&nbsp;';
            $scores2_uid_count  ||= '&nbsp;';
            $scoresNA_uid_count ||= '&nbsp;';
            $scores_uid_count   ||= '&nbsp;';
            $t .= "<td>$scores5_uid_count</td>";
            $t .= "<td>$scores4_uid_count</td>";
            $t .= "<td>$scores3_uid_count</td>";
            $t .= "<td>$scores2_uid_count</td>";
            $t .= "<td>$scoresNA_uid_count</td>";
            $t .= "<td>$scores_uid_count</td>";
            $t .= '</tr>';
        }

    }

    my ( $avgs_html, $avg_score );
    unless ($for_parallel) {
        foreach my $sg (@subjects_with_subgroups) {
            $avgs_html .= '<td>';
            $avgs_html .=
              $avg_count{"$sg->[0]g$sg->[1]"}
              ? sprintf( "%.3f",
                $avg_scores{"$sg->[0]g$sg->[1]"} /
                  $avg_count{"$sg->[0]g$sg->[1]"} )
              : '&nbsp;';
            $avgs_html .= '</td>';
        }
    }

    my ( $quality_html, $qual_by_subj );
    foreach my $sg (@subjects_with_subgroups) {
        $qual_by_subj = $avg_count{"$sg->[0]g$sg->[1]"}
          ? sprintf( "%.1f",
            ( $scores5{"$sg->[0]g$sg->[1]"} + $scores4{"$sg->[0]g$sg->[1]"} ) /
              $avg_count{"$sg->[0]g$sg->[1]"} *
              100 )
          : '&nbsp;';
        $qual_by_subj =~ s/,0+$//o;
        $quality_html .= "<td>$qual_by_subj</td>";
    }

    my ( $advancement_html, $adv_by_subj );
    foreach my $sg (@subjects_with_subgroups) {
        $adv_by_subj = $avg_count{"$sg->[0]g$sg->[1]"}
          ? sprintf(
            "%.1f",
            (
                $scores5{"$sg->[0]g$sg->[1]"} +
                  $scores4{"$sg->[0]g$sg->[1]"} +
                  $scores3{"$sg->[0]g$sg->[1]"}
              ) / $avg_count{"$sg->[0]g$sg->[1]"} * 100
          )
          : '&nbsp;';
        $adv_by_subj =~ s/,0+$//;
        $advancement_html .= "<td>$adv_by_subj</td>";
    }

    my ( $uou_html, $uou_by_subj );
    foreach my $sg (@subjects_with_subgroups) {
        my $uid_count =
            $pupil_count_in_subgrp{ $sg->[1] }
          ? $pupil_count_in_subgrp{ $sg->[1] }
          : scalar(@uids);
        $uou_by_subj = $avg_count{"$sg->[0]g$sg->[1]"}
          ? sprintf(
            "%.1f",
            (
                $scores5{"$sg->[0]g$sg->[1]"} +
                  $scores4{"$sg->[0]g$sg->[1]"} * 0.64 +
                  $scores3{"$sg->[0]g$sg->[1]"} * 0.36 +
                  $scores2{"$sg->[0]g$sg->[1]"} * 0.16
              ) / $uid_count * 100
          )
          : '&nbsp;';
        $uou_by_subj =~ s/,0+$//;
        $uou_html .= "<td>$uou_by_subj</td>";
    }

    my (
        $per_subj_score_count_html,  $score_count_by_subj,
        $per_subj_score5_count_html, $per_subj_score4_count_html,
        $per_subj_score3_count_html, $per_subj_score2_count_html
    );
    unless ($for_parallel) {
        foreach my $sg (@subjects_with_subgroups) {
            $score_count_by_subj = $avg_count{"$sg->[0]g$sg->[1]"} || '&nbsp;';
            $per_subj_score_count_html .= "<td>$score_count_by_subj</td>";
            $score_count_by_subj = $scores5{"$sg->[0]g$sg->[1]"} || '&nbsp;';
            $per_subj_score5_count_html .= "<td>$score_count_by_subj</td>";
            $score_count_by_subj = $scores4{"$sg->[0]g$sg->[1]"} || '&nbsp;';
            $per_subj_score4_count_html .= "<td>$score_count_by_subj</td>";
            $score_count_by_subj = $scores3{"$sg->[0]g$sg->[1]"} || '&nbsp;';
            $per_subj_score3_count_html .= "<td>$score_count_by_subj</td>";
            $score_count_by_subj = $scores2{"$sg->[0]g$sg->[1]"} || '&nbsp;';
            $per_subj_score2_count_html .= "<td>$score_count_by_subj</td>";
        }
    }

    my $default_sou =
      sqlff qq[select f_int from sp_settings where name = 'default_sou'];
    my $sous_html = '';
    my $sou;
    my $req;
    foreach my $sg (@subjects_with_subgroups) {
        $req = $sg->[2] || $default_sou;

        if ( !$avg_count{"$sg->[0]g$sg->[1]"} ) {
            $sous_html .= '<td>&nbsp;</td>';
            next;
        }
        if ( $req == 1 ) {
            $sou =
              ( $scores5{"$sg->[0]g$sg->[1]"} +
                  0.64 * $scores4{"$sg->[0]g$sg->[1]"} +
                  0.36 * $scores3{"$sg->[0]g$sg->[1]"} ) /
              $avg_count{"$sg->[0]g$sg->[1]"};
        }
        if ( $req == 2 ) {
            $sou =
              ( 0.64 * $scores5{"$sg->[0]g$sg->[1]"} +
                  0.36 * $scores4{"$sg->[0]g$sg->[1]"} +
                  0.16 * $scores3{"$sg->[0]g$sg->[1]"} ) /
              $avg_count{"$sg->[0]g$sg->[1]"};
        }
        if ( $req == 3 ) {
            $sou =
              ( 0.36 * $scores5{"$sg->[0]g$sg->[1]"} +
                  0.16 * $scores4{"$sg->[0]g$sg->[1]"} +
                  0.04 * $scores3{"$sg->[0]g$sg->[1]"} ) /
              $avg_count{"$sg->[0]g$sg->[1]"};
        }

        $sous_html .= '<td>' . ( sprintf( "%.2f", $sou ) ) . '</td>';
    }

    my $class_advancement;
    if ( !scalar( keys %avg_count ) and !$na and !scalar( keys %freed_pupils ) )
    {
        $class_advancement = '0.0';
    }
    else {
        $class_advancement = @uids
          ? sprintf "%.1f", ( ( @uids - ( $has2 + $na ) ) / @uids ) * 100
          : '0';
    }

    my $class_quality = @uids
      ? sprintf "%.1f", ( ( $all5 + $all45 + $one4 ) / @uids ) * 100
      : '0';

    my $class_uou = sprintf "%.1f",
      (
        (
            $all5 +
              ( $all45 + $one4 ) * 0.64 +
              ( $one3 + $has3s ) * 0.36 +
              $has2 * 0.16
        ) / scalar(@uids)
      ) * 100;

    if ($for_parallel) {
        @{$out_ref} = ();
        push @{$out_ref}, $all5;
        push @{$out_ref}, $all45;
        push @{$out_ref}, $one4;
        push @{$out_ref}, $one3;
        push @{$out_ref}, $has2;
        push @{$out_ref}, $na;
        push @{$out_ref}, scalar keys %freed_pupils;
        push @{$out_ref}, $class_advancement;
        push @{$out_ref}, $class_quality;
        push @{$out_ref}, $has3s;
        push @{$out_ref}, $has2na;
        push @{$out_ref}, $class_uou;
        return;
    }

    my %replace = (
        menu => generate_menu,

        class  => $resolved_class,
        period => "$resolved_period ($year_total &ndash; "
          . ( $year_total + 1 ) . ')',
        colspan_subjects => scalar(@subjects_with_subgroups),
        subjs            => $subjs_html,
        sorted           => ( param('sort') eq 'avg' )
        ? '<input onclick="turn_sort_avg()" type=checkbox checked><img src="/sp/img/sort.gif" title="    " align=absmiddle>'
        : '<input onclick="turn_sort_avg()" type=checkbox><img src="/sp/img/sort.gif" title=" ,      " align=absmiddle>',
        table                 => $t,
        avg                   => $avgs_html,
        quality               => $quality_html,
        advancement           => $advancement_html,
        per_subj_score_count  => $per_subj_score_count_html,
        per_subj_score5_count => $per_subj_score5_count_html,
        per_subj_score4_count => $per_subj_score4_count_html,
        per_subj_score3_count => $per_subj_score3_count_html,
        per_subj_score2_count => $per_subj_score2_count_html,
        sou                   => $sous_html,
        uou                   => $uou_html,

        all5   => $all5   || 0,
        one4   => $one4   || 0,
        all45  => $all45  || 0,
        one3   => $one3   || 0,
        has3s  => $has3s  || 0,
        has2   => $has2   || 0,
        has2na => $has2na || 0,
        class_quality     => $class_quality,
        class_advancement => $class_advancement,
        na                => $na,
        freed             => scalar( keys %freed_pupils ),
        class_uou         => $class_uou,

        footers => generate_footers,
    );
    tpl( './tpl/report/class.html', \%replace );
}

sub report_parallel_form() {
    my $periods_mode =
      sqlff 'SELECT F_INT FROM SP_SETTINGS WHERE NAME = \'periods_mode\'';
    my $periods;
    if ($periods_mode) {
        for ( 1 .. 4 ) {
            $periods .=
qq[<input type=radio id="idq$_" name="period" value="q$_"><label for="idq$_">$_ <br></label>];
        }
    }
    else {
        for ( 1 .. 3 ) {
            $periods .=
qq[<input type=radio id="idt$_" name="period" value="t$_"><label for="idt$_">$_ <br></label>];
        }
    }

    my $parallels = filter_parallel();

    my $halfs =
      ( sqlff "select f_int from sp_settings where name = 'enable_half_year'" )
      ? q[<input type=radio id="idh1" name="period" value="h1"><label for="idh1">1 <br></label>
<input type=radio id="idh2" name="period" value="h2"><label for="idh2">2 <br></label>]
      : '';

    tpl './tpl/report/parallel_form.html',
      {
        menu      => generate_menu(),
        parallels => $parallels,
        periods   => $periods,
        halfs     => $halfs,
        footers   => generate_footers(),
      };
}

sub report_parallel() {
    my $p = param('id');
    unless ( defined $p ) { stop "   ." }
    else                  { sec_gt0 $p }

    my $period = param('period');

    my @c;
    sqla \@c, "select sys_matrix.class, paralel || class.name from sys_matrix
join class on sys_matrix.class = class.class_id
where view_class = 7
and paralel = $p
and exists
(
  select 1 from get_node_saldo
  (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
)";

    unless ( scalar(@c) ) { stop '    . ' }

    my %b = map { $$_[1] => $$_[0] } @c;
    @c = ();
    push @c, [ $b{$_}, $_ ] for ( sort keys %b );

    my $head_classes = '';
    my ( $all5, $all45, $one4, $one3, $has2, $has3s, $has2na, $na, $freed,
        $class_advancement_html, $class_quality_html, $uou_html );
    my @out;
    my @sums;
    my ( @classes, @adv, @qua, @ouo );
    my ( $pupil_count, $pupil_count_html, $pupil_count_parallel );

    for (@c) {
        $$_[1] =~ s/^(\d+\s{0,}\".+\").+/$1/;
        $head_classes .=
          qq[<th><a href="?act=report&c=$$_[0]&p=$period">$$_[1]</a></th>];
        report( $$_[0], $period, \@out );
        $all5   .= "<td>$out[0]</td>";
        $all45  .= "<td>$out[1]</td>";
        $one4   .= "<td>$out[2]</td>";
        $one3   .= "<td>$out[3]</td>";
        $has2   .= "<td>$out[4]</td>";
        $has3s  .= "<td>$out[9]</td>";
        $has2na .= "<td>$out[10]</td>";
        $na     .= "<td>$out[5]</td>";
        $freed  .= "<td>$out[6]</td>";
        push @classes, str2unicode( $$_[1] );
        push @adv,     $out[7];
        push @qua,     $out[8];
        push @ouo,     $out[11];
        $class_advancement_html .= "<td>$out[7]</td>";
        $class_quality_html     .= "<td>$out[8]</td>";
        $uou_html               .= "<td>$out[11]</td>";
        $pupil_count = sqlff "select count(1) from sys_matrix
where view_class = 1
and class = $$_[0]
and exists
(
  select 1 from get_node_saldo
  (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
)" || 0;
        $pupil_count_html .= "<td>$pupil_count</td>";
        for ( my $i = 0 ; $i < scalar @out ; $i++ ) { $sums[$i] += $out[$i]; }
        $pupil_count_parallel += $pupil_count;
    }
    $pupil_count_html .= "<td><b>$pupil_count_parallel</b></td>";
    $all5             .= "<td><b>$sums[0]</b></td>";
    $all45            .= "<td><b>$sums[1]</b></td>";
    $one4             .= "<td><b>$sums[2]</b></td>";
    $one3             .= "<td><b>$sums[3]</b></td>";
    $has3s            .= "<td><b>$sums[9]</b></td>";
    $has2             .= "<td><b>$sums[4]</b></td>";
    $has2na           .= "<td><b>$sums[10]</b></td>";
    $na               .= "<td><b>$sums[5]</b></td>";
    $freed            .= "<td><b>$sums[6]</b></td>";

    my $parallel_advancement = $pupil_count_parallel
      ? sprintf "%.1f",
      ( ( $pupil_count_parallel - ( $sums[4] + $sums[5] ) ) /
          $pupil_count_parallel ) * 100
      : '0';

    my $parallel_quality = $pupil_count_parallel
      ? sprintf "%.1f",
      ( ( $sums[0] + $sums[1] ) / $pupil_count_parallel ) * 100
      : '0';

    my $parallel_uou = $pupil_count_parallel
      ? sprintf "%.1f",
      (
        (
            $sums[0] +
              $sums[1] * 0.64 +
              ( $sums[3] + $sums[9] ) * 0.36 +
              $sums[4] * 0.16
        ) / $pupil_count_parallel
      ) * 100
      : '0';

    $class_advancement_html .= "<td><b>$parallel_advancement</b></td>";
    $class_quality_html     .= "<td><b>$parallel_quality</b></td>";
    $uou_html               .= "<td><b>$parallel_uou</b></td>";

    $all5   =~ s|>([^0]\d*)<|><span class=sc_all5>$1</span><|g;
    $all45  =~ s|>([^0]\d*)<|><span class=sc_all45>$1</span><|g;
    $one3   =~ s|>([^0]\d*)<|><span class=sc_one3>$1</span><|g;
    $one4   =~ s|>([^0]\d*)<|><span class=sc_one4>$1</span><|g;
    $has2   =~ s|>([^0]\d*)<|><span class=sc_has2>$1</span><|g;
    $has2na =~ s|>([^0]\d*)<|><span class=sc_has2na>$1</span><|g;

    $all5   =~ s|>0<|><span class=sc_grey>0</span><|go;
    $one4   =~ s|>0<|><span class=sc_grey>0</span><|go;
    $all45  =~ s|>0<|><span class=sc_grey>0</span><|go;
    $one3   =~ s|>0<|><span class=sc_grey>0</span><|go;
    $has2   =~ s|>0<|><span class=sc_grey>0</span><|go;
    $has2na =~ s|>0<|><span class=sc_grey>0</span><|go;
    $na     =~ s|>0<|><span class=sc_grey>0</span><|go;
    $freed  =~ s|>0<|><span class=sc_grey>0</span><|go;

    my @data = ( [@classes], [@adv], [@qua], [@ouo] );
    my $my_graph = GD::Graph::bars->new( 80 * ( scalar @classes ) + 170, 300 );
    $my_graph->set(
        title                => ' ',
        long_ticks           => 1,
        y_max_value          => 100,
        y_tick_number        => 10,
        bar_spacing          => 25,
        bargroup_spacing     => 20,
        bar_width            => 24,
        shadow_depth         => 3,
        show_values          => 1,
        values_space         => 2,
        valuesclr            => 'black',
        boxclr               => '#fbfbfb',
        fgclr                => '#c5c5c5',
        accentclr            => 'black',
        shadowclr            => '#aaaaaa',
        textclr              => 'black',
        axislabelclr         => 'black',
        legendclr            => 'black',
        dclrs                => [ '#00ba5e', '#6f97fe', '#e7da8b' ],
        legend_spacing       => 9,
        text_space           => 0,
        legend_marker_width  => 10,
        legend_marker_height => 10,
    );
    $my_graph->set_values_font( 'tahoma.ttf', 9 );
    $my_graph->set_x_axis_font( 'tahoma.ttf', 14 );
    $my_graph->set_y_axis_font( 'tahoma.ttf', 8 );
    $my_graph->set_title_font( 'tahoma.ttf', 20 );
    $my_graph->set_legend_font( 'tahoma.ttf', 11 );
    $my_graph->set_legend(
        str2unicode(' (%)'),
        str2unicode(' (%)'),
        str2unicode(' (%)')
    );
    $my_graph->plot( \@data );
    my $timestamp = time;

    cleanup_tmp('img_report_parallel');

    my $image_file_abs =
      $gconf{sppath} . '/tmp/img_report_parallel/' . $timestamp . '.png';
    my $image_file_rel =
      $gconf{sproot} . '/tmp/img_report_parallel/' . $timestamp . '.png';
    open( OUT, '>', $image_file_abs );
    binmode OUT;
    print OUT $my_graph->gd->png();
    close OUT;

    my @year = year_academic();

    tpl './tpl/report/parallel.html',
      {
        menu         => generate_menu(),
        parallel     => sqlff("select name from paralel where paralel_id = $p"),
        head_classes => $head_classes,
        period       => $g_period_to_name{$period},
        year1        => $year[0],
        year2        => $year[1],
        pupil_count  => $pupil_count_html,
        all5         => $all5,
        all45        => $all45,
        one4         => $one4,
        one3         => $one3,
        has2         => $has2,
        has2na       => $has2na,
        has3s        => $has3s,
        na           => $na,
        freed        => $freed,
        class_advancement => $class_advancement_html,
        class_quality     => $class_quality_html,
        uou               => $uou_html,
        image_file        => $image_file_rel,
        footers           => generate_footers(),
      };
}

sub report_school_form() {
    my $periods_mode =
      sqlff q[select f_int from sp_settings where name = 'periods_mode'];
    my $enable_half_year =
      sqlff "select f_int from sp_settings where name = 'enable_half_year'";

    my $periods;
    my $checked;
    if ($periods_mode) {
        for ( 1 .. 4 ) {
            $checked = $_ == 1 ? 'checked' : '';
            $periods .=
qq[<input $checked type=radio id="idq$_" name="period" value="q$_"><label for="idq$_">$_ <br></label>];
        }
    }
    else {
        for ( 1 .. 3 ) {
            $checked = $_ == 1 ? 'checked' : '';
            $periods .=
qq[<input $checked type=radio id="idt$_" name="period" value="t$_"><label for="idt$_">$_ <br></label>];
        }
    }

    if ($enable_half_year) {
        my $p_mim = sqlff "select min(paralel_id) from paralel";
        my ( $p1, $p2 ) = split /,/,
          sqlff "select f_char from sp_settings where name = 'half_year'";
        tpl './tpl/report/school_form_half.html',
          {
            menu       => generate_menu(),
            periods    => $periods,
            parallels1 => "$p_mim&ndash;" . ( $p1 - 1 ),
            parallels2 => "$p1&ndash;$p2",
            footers    => generate_footers(),
          };
    }
    else {
        tpl './tpl/report/school_form.html',
          {
            menu    => generate_menu(),
            periods => $periods,
            footers => generate_footers(),
          };
    }

}

sub report_school() {

    my $period      = param('period');
    my $period_half = param('period_half');
    my $enable_half_year =
      sqlff "select f_int from sp_settings where name = 'enable_half_year'";
    if ( $period eq 'y' ) { $period_half = 'y'; }
    if ( $period eq 'e' ) { $period_half = 'e'; }
    if ( $period eq 't' ) { $period_half = 't'; }

    stop '   .'
      unless $g_period_to_name{$period};
    if ($enable_half_year) {
        stop '      .'
          unless $g_period_to_name{$period_half};
    }

    {
        my @year_set = sqlf
'select time_start, time_end from sp_periods where link = 0 and id = 0';
        $_ = todate($_) for (@year_set);
        my $time_machine_date = sqlff "select min(sys_entry.entry_date)
from sys_entry
where sys_entry.entry_date between '$year_set[0]' and '$year_set[1]'
  and ( sys_entry.doc_class = 7 or sys_entry.doc_class = 8 )
  and exists(select 1 from sys_matrix where
        sys_entry.credit_node = node_id and
        view_class = 7 and class in (
  select class from sys_matrix
  where sys_matrix.view_class = 7 and
    exists ( select 1 from get_node_saldo
    (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)
  )
)", { ib_timestampformat => "%d.%m.%Y" };

        if ($time_machine_date) {
            next if ( totime($time_machine_date) > totime($gdate) );

            my $tmp = todate day_sub( totime $time_machine_date);
            msg '',
qq[ $gdate.         .<br>  $time_machine_date.<br><br>
<a href="?act=state_date_set&d=$tmp"></a>    $tmp.];
        }
    }

    my ( $year1, $year2 ) = year_academic();

    my ( $half1, $half2 ) = split /,/,
      sqlff q[select f_char from sp_settings where name = 'half_year'];

    my %parallels;
    sqlh \%parallels, "select paralel_id,
(
  select count(1) from sys_matrix
      where view_class = 7
      and paralel = paralel_id
      and exists
      (
        select 1 from get_node_saldo
        (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
      )
)
from paralel";

    my %lessons;
    sqlh \%lessons, "select lesson_id, 1 from lesson";

    my $p;

    my $parallels_html = '';

    for ( sort { $a <=> $b } keys %parallels ) {
        $p = ( $_ >= $half1 ) ? $period_half : $period;
        unless ( $parallels{$_} ) {
            delete $parallels{$_};
            next;
        }
        $parallels_html .=
qq[<th width=33><a href="?act=report_parallel&id=$_&period=$period">$_</a></th>];
    }

    my (
        $scores_uid_count,  $scoresNA_uid_count, $scores5_uid_count,
        $scores4_uid_count, $scores3_uid_count,  $scores2_uid_count
    );

    my ( $all5, $all45, $one4, $one3, $has3s, $has2, $has2na, $na, $freed );
    my (
        $all5_html, $all45_html,  $one4_html, $one3_html, $has3s_html,
        $has2_html, $has2na_html, $na_html,   $freed_html
    );

    my $class_advancement_html;
    my $class_quality_html;
    my $class_uou_html;
    my %freed_pupils;

    my (@sum);

    my ( $q, $ref );
    my @uids;

    my %uids_in_parallels;
    $q = sqlp "select paralel,vw_pupil from sys_matrix
where sys_matrix.class in
(
  select sys_matrix.class from sys_matrix
  where view_class = 7
  and exists
  (
    select 1 from get_node_saldo
    (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
  )
)
and sys_matrix.view_class = 1 and exists
(
  select 1 from get_node_saldo
  (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
)";
    my @a;
    while ( @a = $q->fetchrow_array ) {
        push @{ $uids_in_parallels{ $a[0] } }, $a[1];
    }

    for my $parallel ( sort { $a <=> $b } keys %parallels ) {

        if ($enable_half_year) {
            $p = ( $parallel >= $half1 ) ? $period_half : $period;
        }
        else {
            $p = $period;
        }

        $all5 = $all45 = $one4 = $one3 = $has3s = $na = $freed = $has2 =
          $has2na     = 0;
        %freed_pupils = ();

        @uids = @{ $uids_in_parallels{$parallel} };

        for (@uids) {

            $scores5_uid_count = $scores4_uid_count = $scores3_uid_count =
              $scores2_uid_count = $scores_uid_count = $scoresNA_uid_count = 0;

            $q = sqlp "select $p from sp_scores_totals
                 where uid = $_ and scoreyear = $year1";

            while ( $ref = $q->fetchrow_arrayref() ) {
                if ( $ref->[0] == 5 ) { $scores5_uid_count++; }
                if ( $ref->[0] == 4 ) { $scores4_uid_count++; }
                if ( $ref->[0] == 3 ) { $scores3_uid_count++; }
                if ( $ref->[0] == 2 ) { $scores2_uid_count++; }
                $scores_uid_count++ if ( defined $ref->[0] );
            }

            $q = sqlp "select t,s from sp_marks
where y = $year1 and p = $g_period_to_str{$p} and uid = $_";
            while ( $ref = $q->fetchrow_arrayref() ) {
                if ( $lessons{ $ref->[1] } ) {
                    if ( $ref->[0] == 1 ) {
                        $scoresNA_uid_count++;
                    }
                    if ( $ref->[0] == 2 ) {
                        $freed_pupils{$_}++;
                    }
                }
            }

            if (    $scores_uid_count
                and $scores_uid_count == $scores5_uid_count
                and !$scoresNA_uid_count )
            {
                $all5++;
            }
            elsif ( $scores_uid_count
                and ( $scores_uid_count - $scores5_uid_count == 1 )
                and $scores4_uid_count == 1
                and !$scoresNA_uid_count )
            {
                $one4++;
            }
            elsif (
                $scores_uid_count
                and ( $scores_uid_count ==
                    ( $scores5_uid_count + $scores4_uid_count ) )
                and !$scoresNA_uid_count
              )
            {
                $all45++;
            }
            elsif ( $scores3_uid_count == 1
                and !$scores2_uid_count
                and !$scoresNA_uid_count )
            {
                $one3++;
            }
            elsif ( $scores3_uid_count > 1
                and !$scores2_uid_count
                and !$scoresNA_uid_count )
            {
                $has3s++;
            }
            elsif ( $scores2_uid_count and !$scoresNA_uid_count ) { $has2++ }
            elsif ( $scores2_uid_count and $scoresNA_uid_count )  { $has2na++ }

            $na++ if ( $scoresNA_uid_count and !$scores2_uid_count );

        }

        my $class_advancement;
        if ( !$na and !$scores_uid_count and !scalar( keys %freed_pupils ) ) {
            $class_advancement = '0.0';
        }
        else {
            $class_advancement =
              @uids ? ( ( @uids - ( $has2 + $na ) ) / @uids ) * 100 : '0';
        }

        my $class_quality =
          @uids ? ( ( $all5 + $all45 + $one4 ) / @uids ) * 100 : '0';

        my $class_uou = @uids
          ? (
            (
                $all5 +
                  ( $all45 + $one4 ) * 0.64 +
                  ( $one3 + $has3s ) * 0.36 +
                  $has2 * 0.16
            ) / scalar(@uids)
          ) * 100
          : '0';

        $sum[0] += $all5;
        $sum[1] += $one4;
        $sum[2] += $all45;
        $sum[3] += $one3;
        $sum[4] += $has3s;
        $sum[5] += $has2;
        $sum[6] += $has2na;
        $sum[7] += $na;
        $sum[8] += scalar keys %freed_pupils;

        $sum[9]  += $class_advancement;
        $sum[10] += $class_quality;
        $sum[11] += $class_uou;

        $all5_html   .= "<td>$all5</td>";
        $one4_html   .= "<td>$one4</td>";
        $all45_html  .= "<td>$all45</td>";
        $one3_html   .= "<td>$one3</td>";
        $has3s_html  .= "<td>$has3s</td>";
        $has2_html   .= "<td>$has2</td>";
        $has2na_html .= "<td>$has2na</td>";
        $na_html     .= "<td>$na</td>";
        $freed_html  .= '<td>' . ( scalar keys %freed_pupils ) . '</td>';

        $all5_html   =~ s|>([^0]\d*)<|><span class=sc_all5>$1</span><|go;
        $one4_html   =~ s|>([^0]\d*)<|><span class=sc_one4>$1</span><|go;
        $all45_html  =~ s|>([^0]\d*)<|><span class=sc_all45>$1</span><|og;
        $one3_html   =~ s|>([^0]\d*)<|><span class=sc_one3>$1</span><|go;
        $has2_html   =~ s|>([^0]\d*)<|><span class=sc_has2>$1</span><|go;
        $has2na_html =~ s|>([^0]\d*)<|><span class=sc_has2na>$1</span><|go;

        $all5_html   =~ s|>0<|><span class=sc_grey>0</span><|go;
        $one4_html   =~ s|>0<|><span class=sc_grey>0</span><|go;
        $all45_html  =~ s|>0<|><span class=sc_grey>0</span><|go;
        $one3_html   =~ s|>0<|><span class=sc_grey>0</span><|go;
        $has2_html   =~ s|>0<|><span class=sc_grey>0</span><|go;
        $has2na_html =~ s|>0<|><span class=sc_grey>0</span><|go;

        $class_advancement = sprintf "%.1f", $class_advancement;
        $class_quality     = sprintf "%.1f", $class_quality;
        $class_uou         = sprintf "%.1f", $class_uou;

        $class_advancement_html .= "<td>$class_advancement</td>";
        $class_quality_html     .= "<td>$class_quality</td>";
        $class_uou_html         .= "<td>$class_uou</td>";

    }

    $sum[9] /= scalar keys %parallels;
    $sum[9] = sprintf "%.1f", $sum[9];
    $sum[10] /= scalar keys %parallels;
    $sum[10] = sprintf "%.1f", $sum[10];
    $sum[11] /= scalar keys %parallels;
    $sum[11] = sprintf "%.1f", $sum[11];

    my $count = sqlff "select count(1) from sys_matrix
where sys_matrix.view_class = 1 and exists
(  select 1 from get_node_saldo
  (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
)";

    $all5_html   .= "<td width=40><b>$sum[0]</b></td>";
    $one4_html   .= "<td><b>$sum[1]</b></td>";
    $all45_html  .= "<td><b>$sum[2]</b></td>";
    $one3_html   .= "<td><b>$sum[3]</b></td>";
    $has3s_html  .= "<td><b>$sum[4]</b></td>";
    $has2_html   .= "<td><b>$sum[5]</b></td>";
    $has2na_html .= "<td><b>$sum[6]</b></td>";
    $na_html     .= "<td><b>$sum[7]</b></td>";
    $freed_html  .= "<td><b>$sum[8]</b></td>";

    $all5_html .=
        '<td width=50><b>'
      . ( sprintf "%.1f%", $sum[0] * 100 / $count )
      . '</b></td>';
    $one4_html .=
      '<td><b>' . ( sprintf "%.1f%", $sum[1] * 100 / $count ) . '</b></td>';
    $all45_html .=
      '<td><b>' . ( sprintf "%.1f%", $sum[2] * 100 / $count ) . '</b></td>';
    $one3_html .=
      '<td><b>' . ( sprintf "%.1f%", $sum[3] * 100 / $count ) . '</b></td>';
    $has3s_html .=
      '<td><b>' . ( sprintf "%.1f%", $sum[4] * 100 / $count ) . '</b></td>';
    $has2_html .=
      '<td><b>' . ( sprintf "%.1f%", $sum[5] * 100 / $count ) . '</b></td>';
    $has2na_html .=
      '<td><b>' . ( sprintf "%.1f%", $sum[6] * 100 / $count ) . '</b></td>';
    $na_html .=
      '<td><b>' . ( sprintf "%.1f%", $sum[7] * 100 / $count ) . '</b></td>';
    $freed_html .=
      '<td><b>' . ( sprintf "%.1f%", $sum[8] * 100 / $count ) . '</b></td>';

    $class_advancement_html .= "<td colspan=2><b>$sum[9]</b></td>";
    $class_quality_html     .= "<td colspan=2><b>$sum[10]</b></td>";
    $class_uou_html         .= "<td colspan=2><b>$sum[11]</b></td>";

    my $pupil_count_parallel = '';
    my $pupils_in_school     = 0;
    for ( sort { $a <=> $b } keys %parallels ) {
        $pupil_count_parallel .=
          '<td>' . scalar( @{ $uids_in_parallels{$_} } ) . '</td>';
        $pupils_in_school += scalar( @{ $uids_in_parallels{$_} } );
    }
    $pupil_count_parallel .= "<td colspan=2><b>$pupils_in_school</b></td>";

    my ( @percents, @percents2 );
    for ( 0 .. 7 ) { push @percents, sprintf "%.1f", $sum[$_] * 100 / $count; }

    my @short_labels = (
        '"5"',
        ' "4"',
        '"5"  "4"',
        ' "3"',
        '  "3"',
        '"2"',
        ' "2"  /',
        '/'
    );
    for ( my $i = 0 ; $i < scalar @percents ; $i++ ) {
        if ( $percents[$i] > 8 ) {
            push @percents2,
              str2unicode( $percents[$i] . "%\n" . $short_labels[$i] );
        }
        else { push @percents2, $percents[$i] . '%'; }
    }

    my $sum;
    for (@percents) { $sum += $_; }
    $sum = ( $sum < 100 ) ? ( sprintf( "%.1f", 100 - $sum ) ) . '%' : 0;

    my @data = ( [@percents2], [@percents] );

    if ($sum) {
        push @{ $data[0] }, $sum . str2unicode("\n ");
        push @{ $data[1] }, $sum;
    }

    s/,/./ for @{ $data[1] };

    my $pie = GD::Graph::pie->new( 350, 260 );
    $pie->set(
        axislabelclr   => 'black',
        start_angle    => 90,
        pie_height     => 20,
        transparent    => 1,
        accentclr      => '#555555',
        suppress_angle => 10,
        '3d'           => 1,
        dclrs          => [
            '#9FF4B8', '#D8F49F', '#9FC4F4', '#E6DBCA', '#DEDEDE', '#F8CB71',
            '#FFC3B8', '#F26C50', '#fafafa'
        ]
    );
    $pie->set_text_clr('black');
    $pie->set_title_font( "tahoma.ttf", 14 );
    $pie->set_value_font( "tahoma.ttf", 8 );
    $pie->plot( \@data );

    cleanup_tmp('img_report_school');

    my $timestamp = time;
    my $image_file_abs =
      $gconf{sppath} . '/tmp/img_report_school/' . $timestamp . '.png';
    my $image_file_rel =
      $gconf{sproot} . '/tmp/img_report_school/' . $timestamp . '.png';
    open( OUT, '>', $image_file_abs );
    binmode OUT;
    print OUT $pie->gd->png();
    close OUT;

    my $tmp_half =
      ($enable_half_year)
      ? " &laquo;$g_period_to_name{$period_half}&raquo;"
      : '';
    my $period_line;
    if ( $period eq 'y' ) {
        $period_line = qq[ $year1&ndash;$year2  ];
    }
    else {
        $period_line =
qq[ &laquo;$g_period_to_name{$period}&raquo; $tmp_half $year1&ndash;$year2];
    }

    tpl './tpl/report/school.html ',
      {
        menu              => generate_menu(),
        footers           => generate_footers(),
        period_line       => $period_line,
        ou                => MOUName('short'),
        head_parallels    => $parallels_html,
        pupil_count       => $pupil_count_parallel,
        all5              => $all5_html,
        all45             => $all45_html,
        one4              => $one4_html,
        one3              => $one3_html,
        has3s             => $has3s_html,
        has2              => $has2_html,
        has2na            => $has2na_html,
        na                => $na_html,
        freed             => $freed_html,
        class_advancement => $class_advancement_html,
        class_quality     => $class_quality_html,
        uou               => $class_uou_html,
        image_file        => $image_file_rel,
        legend_add_white  => ($sum)
        ? '<table class=tmp border="0" align=center cellspacing=0><tr><td><div class=legend style="background-color:#fafafa"></div>&nbsp;&nbsp;&nbsp;&nbsp;  </td></tr></table>'
        : '',
      };
}

sub report_form_multiclass() {
    my $filter_class;

    if ( $ggid == 2 ) {
        $filter_class =
          $gteacher_show_all ? filter_class(0) : filter_class_teacher_binded(0);
    }
    else {
        $filter_class = filter_class(0);
    }

    tpl './tpl/report/multiclass_form.html',
      {
        menu         => generate_menu(),
        footers      => generate_footers(),
        filter_class => $filter_class,
      };
}

sub report_links_form() {
    my $periods_mode =
      sqlff 'SELECT F_INT FROM SP_SETTINGS WHERE NAME = \'periods_mode\'';
    my $periods;
    if ($periods_mode) {
        for ( 1 .. 4 ) {
            $periods .=
qq[<input type=radio id="idq$_" name="p" value="q$_"><label for="idq$_">$_ <br></label>];
        }
    }
    else {
        for ( 1 .. 3 ) {
            $periods .=
qq[<input type=radio id="idt$_" name="p" value="t$_"><label for="idt$_">$_ <br></label>];
        }
    }

    my $halfs =
      ( sqlff "select f_int from sp_settings where name = 'enable_half_year'" )
      ? q[<input type=radio id="idh1" name="p" value="h1"><label for="idh1">1 <br></label>
<input type=radio id="idh2" name="p" value="h2"><label for="idh2">2 <br></label>]
      : '';

    tpl './tpl/report/links_form.html',
      {
        menu    => generate_menu(),
        footers => generate_footers(),
        periods => $periods,
        halfs   => $halfs,
      };
}

sub report_links() {
    my @id = param 'id';
    my $p  = param 'p';
    unless (@id) { stop "     ."; }
    unless ($p)  { stop "   ."; }

    my $q;

    my @links = split /,/,
      sqlff q[select f_char from sp_settings where name = 'class_links'];

    my ( $y1, $y2 ) = year_academic();
    my $period_html = "$y1&ndash;$y2";

    my $id_resolved = join ', ', @id;
    my %link_names = (
        0 => '',
        1 => '',
        2 => ''
    );
    $id_resolved =~ s/(\d)/$link_names{$1}/g;

    for ( keys %link_names ) {
        $link_names{$_} = ucfirst $link_names{$_};
    }

    my @c0;
    my @c1;
    my @c2;

    my %pup;

    my %pupils;

    if ( grep /^0$/, @id ) {
        sqlffa \@c0, "select class from sys_matrix
      where view_class = 7
      and paralel between $links[0] and $links[1]
      and exists
      (
        select 1 from get_node_saldo
        (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
      )";
        my $sql = join ',', @c0;
        $pup{'0'} = sqlff qq[select count(1) from sys_matrix
where sys_matrix.view_class = 1
and class in ($sql)
and exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0)
where saldo_sum <> 0 )];
        my @tmp;
        sqlffa \@tmp, qq[select vw_pupil from sys_matrix
where sys_matrix.view_class = 1
and class in ($sql)
and exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0)
where saldo_sum <> 0 )];
        $pupils{'0'} = [@tmp];
    }

    if ( grep /^1$/, @id ) {
        sqlffa \@c1, "select class from sys_matrix
      where view_class = 7
      and paralel between $links[2] and $links[3]
      and exists
      (
        select 1 from get_node_saldo
        (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
      )";
        my $sql = join ',', @c1;
        $pup{'1'} = sqlff qq[select count(1) from sys_matrix
where sys_matrix.view_class = 1
and class in ($sql)
and exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0)
where saldo_sum <> 0 )];
        my @tmp;
        sqlffa \@tmp, qq[select vw_pupil from sys_matrix
where sys_matrix.view_class = 1
and class in ($sql)
and exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0)
where saldo_sum <> 0 )];
        $pupils{'1'} = [@tmp];
    }

    if ( grep /^2$/, @id ) {
        sqlffa \@c2, "select class from sys_matrix
      where view_class = 7
      and paralel between $links[4] and $links[5]
      and exists
      (
        select 1 from get_node_saldo
        (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0
      )";
        my $sql = join ',', @c2;
        $pup{'2'} = sqlff qq[select count(1) from sys_matrix
where sys_matrix.view_class = 1
and class in ($sql)
and exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0)
where saldo_sum <> 0 )];
        my @tmp;
        sqlffa \@tmp, qq[select vw_pupil from sys_matrix
where sys_matrix.view_class = 1
and class in ($sql)
and exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0)
where saldo_sum <> 0 )];
        $pupils{'2'} = [@tmp];
    }

    my $head_links;
    for (@id) {
        $head_links .= qq~<th>$link_names{$_}</th>~;
    }

    if ( scalar @id > 1 ) {
        $head_links .= '<th width=80></th>';
    }

    my $pupil_count;
    for (@id) {
        $pupil_count .= qq[<td>$pup{$_}</td>];
    }

    my @sum;

    if ( scalar @id > 1 ) {
        my $tmp = 0;
        for ( keys %pup ) { $tmp += $pup{$_}; }
        $pupil_count .= qq[<td><b>$tmp</b></td>];
    }

    my ( $year1, $year2 ) = year_academic();

    my ( $all5, $all45, $one4, $one3, $has3s, $has2, $has2na, $na, $freed,
        $class_advancement, $class_quality, $class_uou );
    my (
        $all5_html,              $all45_html,         $one4_html,
        $one3_html,              $has3s_html,         $has2_html,
        $has2na_html,            $na_html,            $freed_html,
        $class_advancement_html, $class_quality_html, $class_uou_html
    );
    my (
        $scores_uid_count,  $scores5_uid_count, $scores4_uid_count,
        $scores3_uid_count, $scores2_uid_count, $scoresNA_uid_count
    );

    my $ref;

    my ( @adv, @qua, @ouo );
    @links = ();

    for my $i (@id) {

        $all5 = $all45 = $one4 = $one3 = $has3s = $na = $freed = $has2 =
          $has2na = 0;

        for ( @{ $pupils{$i} } ) {

            $scores5_uid_count = $scores4_uid_count = $scores3_uid_count =
              $scores2_uid_count = $scores_uid_count = $scoresNA_uid_count = 0;

            $q = sqlp "select $p from sp_scores_totals
                 where uid = $_ and scoreyear = $year1";
            while ( $ref = $q->fetchrow_arrayref() ) {
                if ( defined $ref->[0] ) {
                    if ( $ref->[0] == 5 ) { $scores5_uid_count++; }
                    if ( $ref->[0] == 4 ) { $scores4_uid_count++; }
                    if ( $ref->[0] == 3 ) { $scores3_uid_count++; }
                    if ( $ref->[0] == 2 ) { $scores2_uid_count++; }
                    $scores_uid_count++;
                }
            }

            $q = sqlp "select t,s from sp_marks
where y = $year1 and p = $g_period_to_str{$p} and uid = $_";
            while ( $ref = $q->fetchrow_arrayref() ) {
                if ( $ref->[0] == 1 ) {
                    $scoresNA_uid_count++;
                }
                if ( $ref->[0] == 2 ) {
                    $freed++;
                }
            }

            if (    $scores_uid_count
                and $scores_uid_count == $scores5_uid_count
                and !$scoresNA_uid_count )
            {
                $all5++;
            }
            elsif ( $scores_uid_count
                and ( $scores_uid_count - $scores5_uid_count == 1 )
                and $scores4_uid_count == 1
                and !$scoresNA_uid_count )
            {
                $one4++;
            }
            elsif (
                $scores_uid_count
                and ( $scores_uid_count ==
                    ( $scores5_uid_count + $scores4_uid_count ) )
                and !$scoresNA_uid_count
              )
            {
                $all45++;
            }
            elsif ( $scores3_uid_count == 1
                and !$scores2_uid_count
                and !$scoresNA_uid_count )
            {
                $one3++;
            }
            elsif ( $scores3_uid_count > 1
                and !$scores2_uid_count
                and !$scoresNA_uid_count )
            {
                $has3s++;
            }
            elsif ( $scores2_uid_count and !$scoresNA_uid_count ) { $has2++ }
            elsif ( $scores2_uid_count and $scoresNA_uid_count )  { $has2na++ }

            $na++ if ( $scoresNA_uid_count and !$scores2_uid_count );

        }

        my $class_advancement;
        if ( !$scores_uid_count and !$na and !$freed ) {
            $class_advancement = '0.0';
        }
        else {
            $class_advancement =
              @{ $pupils{$i} }
              ? ( ( @{ $pupils{$i} } - ( $has2 + $na ) ) / @{ $pupils{$i} } ) *
              100
              : '0';
        }

        my $class_quality =
          @{ $pupils{$i} }
          ? ( ( $all5 + $all45 + $one4 ) / @{ $pupils{$i} } ) * 100
          : '0';

        my $class_uou = @{ $pupils{$i} }
          ? (
            (
                $all5 +
                  ( $all45 + $one4 ) * 0.64 +
                  ( $one3 + $has3s ) * 0.36 +
                  $has2 * 0.16
            ) / scalar( @{ $pupils{$i} } )
          ) * 100
          : '0';

        if ( scalar @id > 1 ) {
            $sum[0] += $all5;
            $sum[1] += $one4;
            $sum[2] += $all45;
            $sum[3] += $one3;
            $sum[4] += $has3s;
            $sum[5] += $has2;
            $sum[6] += $has2na;
            $sum[7] += $na;
            $sum[8] += $freed;

            $sum[9]  += $class_advancement;
            $sum[10] += $class_quality;
            $sum[11] += $class_uou;
        }

        $all5_html   .= "<td>$all5</td>";
        $one4_html   .= "<td>$one4</td>";
        $all45_html  .= "<td>$all45</td>";
        $one3_html   .= "<td>$one3</td>";
        $has3s_html  .= "<td>$has3s</td>";
        $has2_html   .= "<td>$has2</td>";
        $has2na_html .= "<td>$has2na</td>";
        $na_html     .= "<td>$na</td>";
        $freed_html  .= "<td>$freed</td>";

        $all5_html   =~ s|>([^0]\d*)<|><span class=sc_all5>$1</span><|go;
        $one4_html   =~ s|>([^0]\d*)<|><span class=sc_one4>$1</span><|go;
        $all45_html  =~ s|>([^0]\d*)<|><span class=sc_all45>$1</span><|og;
        $one3_html   =~ s|>([^0]\d*)<|><span class=sc_one3>$1</span><|go;
        $has2_html   =~ s|>([^0]\d*)<|><span class=sc_has2>$1</span><|go;
        $has2na_html =~ s|>([^0]\d*)<|><span class=sc_has2na>$1</span><|go;

        $all5_html   =~ s|>0<|><span class=sc_grey>0</span><|go;
        $one4_html   =~ s|>0<|><span class=sc_grey>0</span><|go;
        $all45_html  =~ s|>0<|><span class=sc_grey>0</span><|go;
        $one3_html   =~ s|>0<|><span class=sc_grey>0</span><|go;
        $has2_html   =~ s|>0<|><span class=sc_grey>0</span><|go;
        $has2na_html =~ s|>0<|><span class=sc_grey>0</span><|go;

        $class_advancement = sprintf "%.1f", $class_advancement;
        $class_quality     = sprintf "%.1f", $class_quality;
        $class_uou         = sprintf "%.1f", $class_uou;

        $class_advancement_html .= "<td>$class_advancement</td>";
        $class_quality_html     .= "<td>$class_quality</td>";
        $class_uou_html         .= "<td>$class_uou</td>";

        push @links, str2unicode( $link_names{$i} );
        push @adv,   $class_advancement;
        push @qua,   $class_quality;
        push @ouo,   $class_uou;

    }

    if ( scalar @id > 1 ) {
        $all5_html   .= "<td><b>$sum[0]</b></td>";
        $one4_html   .= "<td><b>$sum[1]</b></td>";
        $all45_html  .= "<td><b>$sum[2]</b></td>";
        $one3_html   .= "<td><b>$sum[3]</b></td>";
        $has3s_html  .= "<td><b>$sum[4]</b></td>";
        $has2_html   .= "<td><b>$sum[5]</b></td>";
        $has2na_html .= "<td><b>$sum[6]</b></td>";
        $na_html     .= "<td><b>$sum[7]</b></td>";
        $freed_html  .= "<td><b>$sum[8]</b></td>";

        $sum[9] /= scalar @id;
        $sum[9] = sprintf "%.1f", $sum[9];
        $sum[10] /= scalar @id;
        $sum[10] = sprintf "%.1f", $sum[10];
        $sum[11] /= scalar @id;
        $sum[11] = sprintf "%.1f", $sum[11];

        $class_advancement_html .= "<td colspan=2><b>$sum[9]</b></td>";
        $class_quality_html     .= "<td colspan=2><b>$sum[10]</b></td>";
        $class_uou_html         .= "<td colspan=2><b>$sum[11]</b></td>";
    }

    my @data = ( [@links], [@adv], [@qua], [@ouo] );
    my $my_graph = GD::Graph::bars->new( 80 * ( scalar @links ) + 200, 300 );
    $my_graph->set(
        title            => ' ',
        long_ticks       => 1,
        y_max_value      => 100,
        y_tick_number    => 10,
        bar_spacing      => 25,
        bargroup_spacing => 20,
        bar_width        => 24,
        shadow_depth     => 3,
        show_values      => 1,
        values_space     => 2,
        valuesclr        => 'black',
        boxclr           => '#fbfbfb',
        fgclr            => '#c5c5c5',
        accentclr        => 'black',
        shadowclr        => '#aaaaaa',
        textclr          => 'black',
        axislabelclr     => 'black',
        dclrs            => [ '#00ba5e', '#6f97fe', '#e7da8b' ],
        text_space       => 0,
    );
    $my_graph->set_values_font( 'tahoma.ttf', 9 );
    $my_graph->set_x_axis_font( 'tahoma.ttf', 12 );
    $my_graph->set_y_axis_font( 'tahoma.ttf', 8 );
    $my_graph->set_title_font( 'tahoma.ttf', 14 );
    $my_graph->plot( \@data );
    my $timestamp = time;

    cleanup_tmp('img_report_links');

    my $image_file_abs =
      $gconf{sppath} . '/tmp/img_report_links/' . $timestamp . '.png';
    my $image_file_rel =
      $gconf{sproot} . '/tmp/img_report_links/' . $timestamp . '.png';
    open( OUT, '>', $image_file_abs );
    binmode OUT;
    print OUT $my_graph->gd->png();
    close OUT;

    tpl './tpl/report/links.html',
      {
        menu              => generate_menu(),
        footers           => generate_footers(),
        head_link         => scalar(@id) > 1 ? '' : '',
        period            => $period_html,
        links             => $id_resolved,
        p                 => $g_period_id_to_name{ $g_period_to_str{$p} },
        head_links        => $head_links,
        pupil_count       => $pupil_count,
        all5              => $all5_html,
        all45             => $all45_html,
        one4              => $one4_html,
        one3              => $one3_html,
        has3s             => $has3s_html,
        has2              => $has2_html,
        has2na            => $has2na_html,
        na                => $na_html,
        freed             => $freed_html,
        class_advancement => $class_advancement_html,
        class_quality     => $class_quality_html,
        uou               => $class_uou_html,
        image_file        => $image_file_rel,
      };
}

sub report_teacher_form() {
    my $periods;
    my $periods_mode =
      sqlff q[select f_int from sp_settings where name = 'periods_mode'];
    my $enable_half_year =
      sqlff q[select f_int from sp_settings where name = 'enable_half_year'];

    if ($periods_mode) {
        for ( 1 .. 4 ) {
            $periods .=
qq[<input type=checkbox id="p$_" name="p" value="$_"><label for="p$_"> $_ </label><br>];
        }
    }
    else {
        my $i;
        for ( 1 .. 3 ) {
            $i = $_ + 4;
            $periods .=
qq[<input type=checkbox id="p$i" name="p" value="$i"><label for="p$i"> $_ </label><br>];
        }
    }

    my $halfs = '';
    if ($enable_half_year) {
        for ( 10 .. 11 ) {
            my $half_num = $_ - 9;
            $halfs .=
qq[<input type=checkbox id="p$_" name="p" value="$_"><label for="p$_"> $half_num </label><br>];
        }
    }

    tpl './tpl/report/teacher_form.html',
      {
        menu     => generate_menu(),
        footers  => generate_footers(),
        teachers => filter_teacher($guid),
        periods  => $periods,
        halfs    => $halfs,
        readonly => ( $ggid == 2 and !$ACL{1} ) ? 'disabled' : ''
      };
}

sub report_teacher() {
    my $u = param 'uid';
    if ( $ggid == 2 and !$ACL{1} ) { $u = $guid; }
    stop "   ." unless ($u);
    sec_gt0 $u;

    my @p = param 'p';
    stop "     ." unless (@p);
    for (@p) { sec_range( $_, 0, 12 ); }

    my ( $y1, $y2 ) = year_academic();

    my %score_color_classes = (
        5 => 'sc_all5',
        4 => 'sc_all45',
        3 => 'sc_one3',
        2 => 'sc_has2',
        1 => 'sc_has2na'
    );

    my %class;
    fill_class_ref( \%class );

    my %lesson;
    sqlh \%lesson, "select lesson_id,name from lesson";

    my %class_id_to_parallel;
    sqlh \%class_id_to_parallel, "select class, paralel from sys_matrix
  where sys_matrix.view_class = 7 and
        exists ( select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)";

    my @half =
      split( /,/,
        sqlff("select f_char from sp_settings where name = 'half_year'") );

    my @class_lesson;
    sqla \@class_lesson, qq[select class, lesson, subgrp
from sys_matrix
where view_class = 9001
and vw_teacher = $u
and exists ( select 1 from get_node_saldo_by_unit ( sys_matrix.node_id, '$gdate', '', 0 )
where saldo_sum <> 0)];

    unless ( scalar @class_lesson ) {
        stop q[   
<a href="?act=teacher_placement">&nbsp;</a>
  &nbsp;&nbsp;.];
    }

    my %bind;
    for (@class_lesson) {
        push @{ $bind{ $$_[0] } }, [ $$_[1], $$_[2] ];
    }

    my $t;
    my $pupil_count;
    my %class_cell;

    my %tmp = reverse %g_period_to_str;

    my ( %score, $na, $freed, $class_advancement, $class_quality, $class_uou,
        $style );

    my $sum_pupils_count = 0;
    my $sum_pupils       = 0;

    my %score_sum;
    my $na_sum    = 0;
    my $freed_sum = 0;

    my ( @avg_adv, @avg_qua, @avg_uou, @avg_sou );

    my $default_sou =
      sqlff qq[select f_int from sp_settings where name = 'default_sou'];
    my %sou_hash;
    my $q = sqlp "select c,s,f from sp_req where y = $y1 and g is not null";
    my @a;
    while ( @a = $q->fetchrow_array ) { $sou_hash{"c$a[0]s$a[1]"} = $a[2]; }

    my %tmp_classes_per_subject2;

    my %per_subj_sum_pupils;
    my %per_subj_sum_scores;
    my ( %per_subj_sum_na, %per_subj_sum_freed );
    my (
        %per_subj_avg_adv, %per_subj_avg_qua,
        %per_subj_avg_uou, %per_subj_avg_sou
    );

    my %per_class_pupil_sum;

    my %class_link;

    my $tmp_grp_name;

    for ( sort { int( $class{$a} ) <=> int( $class{$b} ) } keys %bind ) {
        for my $s ( @{ $bind{$_} } ) {
            for my $p (@p) {

                unless ( $class_link{$_} ) {
                    $class_link{$_} = get_class_link($_);
                }

                my @pupils = ();

                if ( $$s[1] ) {
                    sqlffa \@pupils,
                      qq[select uid from sp_class_subgrp where id = $$s[1]];
                }
                else {

                    my $tmp_p = $p;
                    if ( $p == 8 or $p == 9 ) { $tmp_p = 0; }

                    my $period_end = todate sqlff
"select time_end from sp_periods where link = $class_link{$_} and id = $tmp_p";
                    if ($state_date) {
                        $period_end = move_date_to_time_machine($period_end);
                    }

                    sqlffa \@pupils, qq[select vw_pupil from sys_matrix
where sys_matrix.view_class = 1
and sys_matrix.class = $_
and exists(select 1 from get_node_saldo(sys_matrix.node_id, '$period_end', 0) where saldo_sum <> 0)];

                }

                $pupil_count = scalar @pupils;

                unless ( $per_class_pupil_sum{$_} ) {
                    $per_class_pupil_sum{ "p$p" . "c$_" } = scalar @pupils;
                }

                %score = ();
                $na = $freed = 0;

                for my $uid (@pupils) {

                    $q = sqlp "select $tmp{$p} from sp_scores_totals
where scoreyear = $y1 and subj = $$s[0] and uid = $uid";
                    my $ref;
                    while ( $ref = $q->fetchrow_arrayref() ) {
                        $score{ $ref->[0] }++ if $ref->[0];
                    }

                    $q = sqlp "select t from sp_marks
where y = $y1 and p = $p and s = $$s[0] and uid = $uid";
                    while ( $ref = $q->fetchrow_arrayref() ) {
                        if ( $ref->[0] == 1 ) { $na++; }
                        if ( $ref->[0] == 2 ) { $freed++; }
                    }

                }

                for ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
                    $score_sum{$_} += $score{$_};
                    $per_subj_sum_scores{ $$s[0] . 's' . $_ } += $score{$_};
                }

                $na_sum    += $na;
                $freed_sum += $freed;

                $t .= qq[<tr>];

                unless ( $class_cell{$_} ) {
                    $class_cell{$_}++;
                    $t .=
                        q[<td rowspan=]
                      . ( scalar( @{ $bind{$_} } ) * scalar(@p) )
                      . qq[>$class{$_}</td>];
                }

                my $subgrp_style_addon =
                  ( $$s[1] ) ? 'border-left:6px solid #FFF1BA;' : '';

                $t .=
qq[<td style="text-align:left; padding-left: 4px; $subgrp_style_addon">$lesson{$$s[0]}];
                if ( $$s[1] ) {
                    $tmp_grp_name = sqlff
                      "select name from sp_class_subgrp_ref where id = $$s[1]";
                    $t .= qq[<br><span class=subgrp>$tmp_grp_name</span>];
                }
                $t .= qq[</td>];

                $t .=
qq[<td style="text-align:left">$g_period_to_name{$tmp{$p}}</td>]
                  unless scalar(@p) == 1;

                $t .= qq[<td style="text-align:center">$pupil_count</td>];

                $tmp_classes_per_subject2{ $$s[0] }{$_}++;

                push @{ $per_subj_sum_pupils{ $$s[0] } }, $pupil_count;
                $per_subj_sum_na{ $$s[0] }    += $na;
                $per_subj_sum_freed{ $$s[0] } += $freed;

                $sum_pupils += $pupil_count;
                $sum_pupils_count++;

                my $class_advancement =
                  scalar(@pupils)
                  ? ( $score{'5'} + $score{'4'} + $score{'3'} ) * 100 /
                  scalar(@pupils)
                  : 0;

                my $class_quality =
                  scalar(@pupils)
                  ? ( $score{'5'} + $score{'4'} ) * 100 / scalar(@pupils)
                  : '0';

                my $class_uou =
                  scalar(@pupils)
                  ? ( ( $score{'5'} * 100 ) +
                      ( $score{'4'} * 64 ) +
                      ( $score{'3'} * 36 ) +
                      ( $score{'2'} * 16 ) +
                      ( $na * 7 ) ) / ( scalar(@pupils) - $freed )
                  : '0';

                my $score_count = $na;
                for ( values %score ) {
                    $score_count += $_;
                }

                $na    ||= '&nbsp;';
                $freed ||= '&nbsp;';

                my $sou;
                my $req = $sou_hash{ 'c' . $_ . 's' . $$s[0] } || $default_sou;

                unless ($score_count) {
                    $sou = '&mdash;';
                }
                else {
                    if ( $req == 1 ) {
                        $sou =
                          ( $score{'5'} +
                              0.64 * $score{'4'} +
                              0.36 * $score{'3'} ) / $score_count;
                    }
                    elsif ( $req == 2 ) {
                        $sou =
                          ( 0.64 * $score{'5'} +
                              0.36 * $score{'4'} +
                              0.16 * $score{'3'} ) / $score_count;
                    }
                    elsif ( $req == 3 ) {
                        $sou =
                          ( 0.36 * $score{'5'} +
                              0.16 * $score{'4'} +
                              0.04 * $score{'3'} ) / $score_count;
                    }
                }

                push @avg_adv, $class_advancement if $class_advancement;
                push @avg_qua, $class_quality     if $class_quality;
                push @avg_uou, $class_uou         if $class_uou;
                push @avg_sou, $sou               if $sou;

                push @{ $per_subj_avg_adv{ $$s[0] } }, $class_advancement
                  if $class_advancement;
                push @{ $per_subj_avg_qua{ $$s[0] } }, $class_quality
                  if $class_quality;
                push @{ $per_subj_avg_uou{ $$s[0] } }, $class_uou if $class_uou;
                push @{ $per_subj_avg_sou{ $$s[0] } }, $sou       if $sou;

                $class_advancement = sprintf "%.1f", $class_advancement
                  if $class_advancement;
                $class_quality = sprintf "%.1f", $class_quality
                  if $class_quality;
                $class_uou = sprintf "%.1f", $class_uou if $class_uou;
                $sou       = sprintf "%.2f", $sou       if $sou;

                my $zero_grey = '<span class=sc_grey>0</span>';

                $class_advancement = $zero_grey unless $class_advancement;
                $class_quality     = $zero_grey unless $class_quality;
                $class_uou         = $zero_grey unless $class_uou;
                $sou               = $zero_grey unless $sou;

                my $scores_exist = 0;
                for ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
                    $scores_exist++ if $score{$_};
                }

                if ($scores_exist) {
                    for ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
                        $style = '';
                        for my $key ( keys %score_color_classes ) {
                            $style = " class=$score_color_classes{$key}"
                              if ( $score{$_} and $_ == $key );
                        }
                        $score{$_} = "<b>$score{$_}</b>" if $score{$_};
                        $score{$_} ||= '&nbsp;';
                        $t .= qq[<td$style>$score{$_}</td>];
                    }
                }
                else {
                    my $score_colspan =
                      $gconf{'score_max'} - $gconf{'score_min'} + 1;
                    $t .=
qq[<td colspan=$score_colspan style="color:#aaa;font-size:9px;text-align:center"> </td>];
                }

                $na =
                    '<span '
                  . ( $na ? 'class=sc_has2na' : '' ) . '>'
                  . $na
                  . '</span>';

                $t .= qq[
<td>$na</td>
<td>$freed</td>
<td>$class_advancement</td>
<td>$class_quality</td>
<td>$class_uou</td>
<td>$sou</td>
</tr>];

            }
        }
    }

    my @tmp_subjects;
    my %uniq_subj;

    for my $b (%bind) {
        my $tmp = $bind{$b};
        for ( @{$tmp} ) {
            unless ( $uniq_subj{ $$_[0] } ) {
                $uniq_subj{ $$_[0] }++;
                push @tmp_subjects, $lesson{ $$_[0] };
            }
        }
    }
    my $subjects = join ', ', @tmp_subjects;

    my $colspan2 =
      ( $gconf{'score_max'} -
          $gconf{'score_min'} + 11 -
          ( ( scalar(@p) == 1 ) ? 1 : 0 ) );

    my $dummy_cell_if_period = ( scalar(@p) == 1 ) ? '' : '<td>&nbsp;</td>';

    my $colspan3 = $gconf{'score_max'} - $gconf{'score_min'} + 3;

    my $totals_per_subject = qq[<tr class=light_grey>
  <td style="text-align:center; height:24px;" colspan="$colspan2">  </td>
</tr>
<tr>
<td><b>-</b></td>
<td style="text-align:center"><b></b></td>
$dummy_cell_if_period
<td style="text-align:center"><b> </b></td>
<td style="text-align:center" colspan="$colspan3"><b></b></td>
<td style="text-align:center" colspan="4"><b> </b></td>
</tr>];

    my $classes_per_subject;

    for ( keys %per_subj_sum_pupils ) {
        $totals_per_subject .= qq[<tr>];

        $classes_per_subject = 0;
        for ( keys %{ $tmp_classes_per_subject2{$_} } ) {
            $classes_per_subject++;
        }

        my $classes_text =
          declension( $classes_per_subject, [ '', '', '' ] );

        $totals_per_subject .= qq[<td>$classes_per_subject $classes_text</td>
         <td style="text-align:left">$lesson{$_}</td>];

        $totals_per_subject .= qq[<td>&nbsp;</td>]
          unless scalar(@p) == 1;

        $totals_per_subject .=
            q[<td style="text-align:center">]
          . avg( $per_subj_sum_pupils{$_} )
          . q[</td>];

        for my $x ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
            $style = '';
            for my $key ( keys %score_color_classes ) {
                $style = " class=$score_color_classes{$key}" if $x == $key;
            }
            my $tmp = $per_subj_sum_scores{ $_ . 's' . $x };
            $tmp ||= '&nbsp;';
            $totals_per_subject .= qq[<td$style>$tmp</td>];
        }

        $per_subj_sum_na{$_}    ||= '&nbsp;';
        $per_subj_sum_freed{$_} ||= '&nbsp;';

        $totals_per_subject .=
            qq[<td>$per_subj_sum_na{$_}</td>]
          . qq[<td>$per_subj_sum_freed{$_}</td>]
          . qq[<td>]
          . avg( $per_subj_avg_adv{$_}, 1 )
          . q[</td>]
          . qq[<td>]
          . avg( $per_subj_avg_qua{$_}, 1 )
          . q[</td>]
          . qq[<td>]
          . avg( $per_subj_avg_uou{$_}, 1 )
          . q[</td>]
          . qq[<td>]
          . avg( $per_subj_avg_sou{$_}, 2 )
          . q[</td>];

        $totals_per_subject .= qq[</tr>];
    }

    my $totals;

    $totals =
        '<td style="text-align:center"><b>'
      . scalar( keys %bind )
      . '</b></td>'
      . '<td style="text-align:center"><b>'
      . scalar(@tmp_subjects)
      . '</b></td>';
    $totals .= '<td>&nbsp;</td>' unless scalar(@p) == 1;
    $totals .=
        qq[<td style="text-align:center"><b>]
      . int( $sum_pupils / $sum_pupils_count )
      . '</b></td>';

    for ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
        $style = '';
        if ( $score_sum{$_} ) {
            for my $key ( keys %score_color_classes ) {
                $style = " class=$score_color_classes{$key}" if $_ == $key;
            }
        }
        $score_sum{$_} ||= '&nbsp;';
        $totals .= qq[<td$style><b>] . $score_sum{$_} . '</b></td>';
    }

    my $paint_na = $na_sum ? ' class=sc_has2na' : '';

    $na_sum    ||= '&nbsp;';
    $freed_sum ||= '&nbsp;';

    $totals .=
        "<td$paint_na><b>$na_sum</b></td>"
      . "<td><b>$freed_sum</b></td>"
      . '<td><b>'
      . avg( \@avg_adv, 1 )
      . '</b></td>'
      . '<td><b>'
      . avg( \@avg_qua, 1 )
      . '</b></td>'
      . '<td><b>'
      . avg( \@avg_uou, 1 )
      . '</b></td>'
      . '<td><b>'
      . avg( \@avg_sou, 2 )
      . '</b></td>';

    my %per_period_pupil_sum;
    my $total_pupil_count = 0;
    for ( keys %per_class_pupil_sum ) {
        /^p(\d+)c/;
        $per_period_pupil_sum{$1} += $per_class_pupil_sum{$_};
    }
    my $per_period_pupil_count;
    for ( sort keys %per_period_pupil_sum ) {
        $per_period_pupil_count .=
"<tr><td>$g_period_to_name{$tmp{$_}}</td><td>$per_period_pupil_sum{$_}</td></tr>";
    }

    my $total_score_count = 0;
    for ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
        $total_score_count += $score_sum{$_};
    }
    $total_score_count += $na_sum;

    my $timestamp = time;

    my $image_file_rel;
    if ($total_score_count) {
        my %percents;
        for ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
            $percents{$_} = sprintf "%.1f",
              $score_sum{$_} * 100 / $total_score_count;
        }
        my $percents_na = sprintf "%.1f", $na_sum * 100 / $total_score_count;

        my @score_sums;
        my @score_titles;
        for ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
            push @score_sums,   $score_sum{$_};
            push @score_titles, "$score_sum{$_} &ndash; $percents{$_}%";
        }
        push @score_sums,   $na_sum;
        push @score_titles, "$na_sum &ndash; $percents_na%";

        my @data = ( [@score_titles], [@score_sums] );

        my $pie = GD::Graph::pie->new( 270, 220 );
        $pie->set(
            axislabelclr   => 'black',
            start_angle    => 90,
            pie_height     => 15,
            transparent    => 1,
            accentclr      => '#555555',
            suppress_angle => 10,
            '3d'           => 1,
            dclrs          => [
                '#DEDEDE', '#F8CB71', '#E6DBCA', '#9FC4F4',
                '#9FF4B8', '#F26C50'
            ]
        );
        $pie->set_text_clr('black');
        $pie->set_value_font( "futura.ttf", 9 );
        $pie->plot( \@data );

        cleanup_tmp('img_report_teacher');

        my $image_file_abs =
          $gconf{sppath} . '/tmp/img_report_teacher/p-' . $timestamp . '.png';
        $image_file_rel =
          $gconf{sproot} . '/tmp/img_report_teacher/p-' . $timestamp . '.png';
        open( OUT, '>', $image_file_abs );
        binmode OUT;
        print OUT $pie->gd->png();
        close OUT;
    }
    else {
        $image_file_rel = "$gconf{'sproot'}/img/forbidden.gif";
    }

    my $total_score_status_count = $total_score_count + $freed_sum;

    my @chart_titles;

    my @data_per_subj_avg_adv;
    my @data_per_subj_avg_qua;
    my @data_per_subj_avg_uou;

    for ( keys %per_subj_sum_pupils ) {
        push @data_per_subj_avg_adv, avg( $per_subj_avg_adv{$_}, 1 );
        push @data_per_subj_avg_qua, avg( $per_subj_avg_qua{$_}, 1 );
        push @data_per_subj_avg_uou, avg( $per_subj_avg_uou{$_}, 1 );
        push @chart_titles, str2unicode( substr( $lesson{$_}, 0, 10 ) );
    }
    if ( scalar(@tmp_subjects) > 2 ) {
        push @data_per_subj_avg_adv, avg( \@avg_adv, 1 );
        push @data_per_subj_avg_qua, avg( \@avg_qua, 1 );
        push @data_per_subj_avg_uou, avg( \@avg_uou, 1 );
        push @chart_titles,          str2unicode('');
    }

    my $image_file_rel2;
    if ($total_score_count) {
        my @data = (
            [@chart_titles], [@data_per_subj_avg_adv],
            [@data_per_subj_avg_qua], [@data_per_subj_avg_uou]
        );
        my $my_graph =
          GD::Graph::bars->new( 210 + ( @chart_titles - 1 ) * 85, 280 );
        $my_graph->set(
            title                => ' ',
            long_ticks           => 1,
            y_max_value          => 100,
            y_tick_number        => 10,
            bar_width            => 12,
            bar_spacing          => 0,
            bargroup_spacing     => 10,
            show_values          => 1,
            values_space         => 1,
            valuesclr            => 'black',
            boxclr               => '#fbfbfb',
            fgclr                => '#c5c5c5',
            accentclr            => 'black',
            shadowclr            => '#aaaaaa',
            textclr              => 'black',
            axislabelclr         => 'black',
            legendclr            => 'black',
            dclrs                => [ '#00ba5e', '#6f97fe', '#e7da8b' ],
            legend_spacing       => 8,
            legend_marker_width  => 10,
            legend_marker_height => 10,
        );

        $my_graph->set_values_font( 'futura.ttf', 9 );
        $my_graph->set_x_axis_font( 'futura.ttf', 8 );
        $my_graph->set_y_axis_font( 'tahoma.ttf', 8 );
        $my_graph->set_title_font( 'tahoma.ttf', 8 );
        $my_graph->set_legend_font( 'tahoma.ttf', 9 );
        $my_graph->set_legend(
            str2unicode(' (%)'),
            str2unicode(' (%)'),
            str2unicode(' (%)')
        );
        $my_graph->plot( \@data );

        my $image_file_abs2 =
          $gconf{sppath} . '/tmp/img_report_teacher/b-' . $timestamp . '.png';
        $image_file_rel2 =
          $gconf{sproot} . '/tmp/img_report_teacher/b-' . $timestamp . '.png';
        open( OUT, '>', $image_file_abs2 );
        binmode OUT;
        print OUT $my_graph->gd->png();
        close OUT;
    }
    else {
        $image_file_rel2 = "$gconf{'sproot'}/img/pixel.gif";
    }

    my $score_head = '';
    for ( $gconf{'score_min'} .. $gconf{'score_max'} ) {
        $score_head .= qq[<th width=25>$_</th>];
    }

    my $br = scalar(@tmp_subjects) > 2;

    tpl './tpl/report/teacher.html',
      {
        menu    => generate_menu(),
        footers => generate_footers(),
        teacher => sqlff(
"select last_name ||' '|| first_name ||' '|| middle_name from human where human_id = $u"
        ),
        subjects   => $subjects,
        score_head => $score_head,
        table      => $t,
        colspan_1  => ( $gconf{'score_max'} - $gconf{'score_min'} + 3 ),
        colspan_2  => $colspan2,
        totals     => $totals,
        totals_per_subject =>
          ( ( scalar(@tmp_subjects) > 1 ) ? $totals_per_subject : '' ),
        single_period => (
            ( scalar(@p) == 1 )
            ? "<br>: $g_period_to_name{$tmp{$p[0]}} ($y1&mdash;$y2)"
            : "<br>: $y1&mdash;$y2"
        ),
        head_period =>
          ( ( scalar(@p) == 1 ) ? '' : '<th rowspan=2></th>' ),
        per_period_pupil_count => $per_period_pupil_count,
        pie                    => $image_file_rel,
        dia1                   => (
            $br
            ? qq[<div align=center style="margin-bottom:16px;"><img src="$image_file_rel2"></div>]
            : ''
        ),
        dia2 => ( $br ? '' : qq[<img src="$image_file_rel2">] ),
      };
}

sub report_progress_movement_form() {
    my $periods;
    my $periods_mode =
      sqlff q[select f_int from sp_settings where name = 'periods_mode'];

    if ($periods_mode) {
        for ( 1 .. 4 ) {
            $periods .=
qq[<input type=radio id="p$_" name="period" value="$_"><label for="p$_"> $_ </label><br>];
        }
    }
    else {
        my $i;
        for ( 1 .. 3 ) {
            $i = $_ + 4;
            $periods .=
qq[<input type=radio id="p$i" name="period" value="$i"><label for="p$i"> $_ </label><br>];
        }
    }

    my $t = sqlff "select f_char from sp_settings where name = 'class_links'";
    $t =~ /(\d+),(\d+),(\d+),(\d+),(\d+),(\d+)/;
    my $auto_parallel_ranges = "$1-$2\\n$3-$4\\n$5-$6";

    tpl './tpl/report/progress_movement_form.html',
      {
        periods              => $periods,
        menu                 => generate_menu(),
        footers              => generate_footers(),
        select_classes       => filter_class(0),
        select_parallels     => filter_parallel(),
        auto_parallel_ranges => $auto_parallel_ranges,
      };

}

sub report_progress_movement() {
    my %c;

    my @c = param('c');
    my @p = param('p');
    my @l = param('l');

    my $period = param('period');
    if ( $period or $period eq '0' ) { sec_range( $period, 0, 12 ); }
    else                             { stop "   ."; }

    for (@c) {
        sec_gt0 $_;
        $c{$_}++;
    }

    my @pl;
    if ( scalar @l and scalar @l <= 3 ) {
        my @a = split( ',',
            sqlff("select f_char from sp_settings where name = 'class_links'")
        );
        for (@l) {
            if ( $_ == 0 ) { push @pl, ( $a[0] .. $a[1] ); }
            if ( $_ == 1 ) { push @pl, ( $a[2] .. $a[3] ); }
            if ( $_ == 2 ) { push @pl, ( $a[4] .. $a[5] ); }
        }
    }

    push @p, @pl;

    if ( scalar @p ) {
        sex_gteq0(@p);

        my $tmp = join ',', @p;
        my @a;
        sqlffa \@a, "select sys_matrix.class from sys_matrix where
sys_matrix.view_class = 7
and paralel in ($tmp)
and exists ( select 1 from get_node_saldo(sys_matrix.node_id,
'$gdate', 0) where saldo_sum <> 0 )";

        for (@a) {
            $c{$_}++;
        }
    }

    my %class_hash;
    fill_class_ref( \%class_hash );

    my @classes;
    for ( keys %c ) {
        push @classes, $_;
    }
    @classes = sort {
        int( $class_hash{$a} ) <=> int( $class_hash{$b} )
          || substr( $class_hash{$a}, 2 ) cmp substr( $class_hash{$b}, 2 )
    } @classes;

    my $head_classes = '';
    for (@classes) {
        $head_classes .= qq[<th nowrap>$class_hash{$_}</th>];
    }

    my @pupils_on_today;
    my @pupils_on_period_begin;
    my @pupils_on_period_end;

    my $main_table = '<tr><td class=l>- </td>';
    for (@classes) {
        my $tmp = sqlff "select count(1) from sys_matrix
                      where sys_matrix.class = $_ and
                            sys_matrix.view_class = 1 and
                            exists(select 1 from get_node_saldo(sys_matrix.node_id,
                              '$gdate', 0) where saldo_sum <> 0)";
        $main_table .= qq[<td>$tmp</td>];
        push @pupils_on_today, $tmp;
    }
    $main_table .= '<td><b>' . ( sum \@pupils_on_today ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l>  </td>';

    my $unixtime_start;
    my $unixtime_end;

    for (@classes) {
        my $link = get_class_link($_);

        ( $unixtime_start, undef ) =
          sqlf "select time_start, time_end from sp_periods
      where link = $link and id = $period";
        my $time_start = todate $unixtime_start;

        my $tmp = sqlff "select count(1) from sys_matrix
                      where sys_matrix.class = $_ and
                            sys_matrix.view_class = 1 and
                            exists(select 1 from get_node_saldo(sys_matrix.node_id,
                              '$time_start', 0) where saldo_sum <> 0)";
        $main_table .= qq[<td>$tmp</td>];
        push @pupils_on_period_begin, $tmp;
    }
    $main_table .=
      '<td><b>' . ( sum \@pupils_on_period_begin ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l>  </td>';

    for (@classes) {
        my $link = get_class_link($_);

        ( undef, $unixtime_end ) =
          sqlf "select time_start, time_end from sp_periods
      where link = $link and id = $period";
        my $time_end = todate $unixtime_end;

        my $tmp = sqlff "select count(1) from sys_matrix
                      where sys_matrix.class = $_ and
                            sys_matrix.view_class = 1 and
                            exists(select 1 from get_node_saldo(sys_matrix.node_id,
                              '$time_end', 0) where saldo_sum <> 0)";
        $main_table .= qq[<td>$tmp</td>];
        push @pupils_on_period_end, $tmp;
    }
    $main_table .=
      '<td><b>' . ( sum \@pupils_on_period_end ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l></td>';
    my @sum_pupil_come;
    for ( my $i = 0 ; $i < @classes ; $i++ ) {
        if ( $pupils_on_period_end[$i] - $pupils_on_period_begin[$i] > 0 ) {
            $main_table .= '<td>'
              . ( $pupils_on_period_end[$i] - $pupils_on_period_begin[$i] )
              . '</td>';
            push @sum_pupil_come,
              ( $pupils_on_period_end[$i] - $pupils_on_period_begin[$i] );
        }
        else {
            $main_table .= qq[<td>&nbsp;</td>];
        }
    }
    $main_table .= '<td><b>' . ( sum \@sum_pupil_come ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l></td>';
    my @sum_pupil_gone;
    for ( my $i = 0 ; $i < @classes ; $i++ ) {
        if ( $pupils_on_period_begin[$i] - $pupils_on_period_end[$i] > 0 ) {
            $main_table .= '<td>'
              . ( $pupils_on_period_begin[$i] - $pupils_on_period_end[$i] )
              . '</td>';
            push @sum_pupil_gone,
              ( $pupils_on_period_begin[$i] - $pupils_on_period_end[$i] );
        }
        else {
            $main_table .= qq[<td>&nbsp;</td>];
        }
    }
    $main_table .= '<td><b>' . ( sum \@sum_pupil_gone ) . '</b></td></tr>';

    my (
        @out,   @adv,  @qua,  @uou,   @pupil_count, @all5,
        @all45, @one4, @one3, @has3s, @has2na,      @freed
    );

    my %periods_to_str_hash = reverse %g_period_to_str;
    for (@classes) {
        report( $_, $periods_to_str_hash{$period}, \@out );
        push @adv, $out[7];
        push @qua, $out[8];
        push @uou, $out[11];

        push @all5,   $out[0];
        push @all45,  $out[1];
        push @one4,   $out[2];
        push @one3,   $out[3];
        push @has3s,  $out[9];
        push @has2na, $out[10];
        push @freed,  $out[6];
    }

    $main_table .= '<tr><td class=l>% </td>';
    for (@adv) {
        $main_table .= qq[<td>$_</td>];
    }
    $main_table .= '<td><b>' . ( avg \@adv, 1 ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l>%  </td>';
    for (@qua) {
        $main_table .= qq[<td>$_</td>];
    }
    $main_table .= '<td><b>' . ( avg \@qua, 1 ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l>% </td>';
    for (@uou) {
        $main_table .= qq[<td>$_</td>];
    }
    $main_table .= '<td><b>' . ( avg \@uou, 1 ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l>"5"</td>';
    for (@all5) {
        if   ($_) { $main_table .= qq[<td>$_</td>]; }
        else      { $main_table .= qq[<td>&nbsp;</td>]; }
    }
    $main_table .= '<td><b>' . ( sum \@all5, 1 ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l>  "4"</td>';
    for (@one4) {
        if   ($_) { $main_table .= qq[<td>$_</td>]; }
        else      { $main_table .= qq[<td>&nbsp;</td>]; }
    }
    $main_table .= '<td><b>' . ( sum \@one4, 1 ) . '</b></td>';
    $main_table .= '</tr>';

    $main_table .= '<tr><td class=l>"4"  "5"</td>';
    for (@all45) {
        if   ($_) { $main_table .= qq[<td>$_</td>]; }
        else      { $main_table .= qq[<td>&nbsp;</td>]; }
    }
    $main_table .= '<td><b>' . ( sum \@all45, 1 ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l>  "3"</td>';
    for (@one3) {
        if   ($_) { $main_table .= qq[<td>$_</td>]; }
        else      { $main_table .= qq[<td>&nbsp;</td>]; }
    }
    $main_table .= '<td><b>' . ( sum \@one3, 1 ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l>  "3"</td>';
    for (@has3s) {
        if   ($_) { $main_table .= qq[<td>$_</td>]; }
        else      { $main_table .= qq[<td>&nbsp;</td>]; }
    }
    $main_table .= '<td><b>' . ( sum \@has3s, 1 ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l> </td>';
    for (@has2na) {
        if   ($_) { $main_table .= qq[<td>$_</td>]; }
        else      { $main_table .= qq[<td>&nbsp;</td>]; }
    }
    $main_table .= '<td><b>' . ( sum \@has2na, 1 ) . '</b></td></tr>';

    $main_table .= '<tr><td class=l></td>';
    for (@freed) {
        if   ($_) { $main_table .= qq[<td>$_</td>]; }
        else      { $main_table .= qq[<td>&nbsp;</td>]; }
    }
    $main_table .= '<td><b>' . ( sum \@freed, 1 ) . '</b></td></tr>';

    if ( $period != 8 and $period != 9 ) {
        my $class_list = join ',', @classes;
        my @skipped_lessons;
        sqla \@skipped_lessons, "select cast(class as integer), score
 from  sp_scores, sys_matrix
where  sys_matrix.vw_pupil = sp_scores.uid
  and  scoretime between $unixtime_start and $unixtime_end
  and  score in (-1,-2)
  and  sys_matrix.class in ($class_list)
  and  exists
  (
    select 1 from get_node_saldo (sys_matrix.node_id, '$gdate', 0)
    where saldo_sum <> 0
  )";

        my $skipped;
        my @skipped_per_class;

        $main_table .= '<tr><td class=l> </td>';
        for (@classes) {
            $skipped = 0;
            for my $i (@skipped_lessons) {
                if ( $_ == $$i[0] ) { $skipped++; }
            }
            if   ($skipped) { $main_table .= qq[<td>$skipped</td>]; }
            else            { $main_table .= qq[<td>&nbsp;</td>]; }
            push @skipped_per_class, $skipped;
        }
        $main_table .=
          '<td><b>' . ( sum \@skipped_per_class, 1 ) . '</b></td></tr>';

        $main_table .= '<tr><td class=l> . </td>';
        my @skipped_per_class_good_reason;
        for (@classes) {
            $skipped = 0;
            for my $i (@skipped_lessons) {
                if ( $_ == $$i[0] and $$i[1] == -2 ) { $skipped++; }
            }
            if   ($skipped) { $main_table .= qq[<td>$skipped</td>]; }
            else            { $main_table .= qq[<td>&nbsp;</td>]; }
            push @skipped_per_class_good_reason, $skipped;
        }
        $main_table .=
            '<td><b>'
          . ( sum \@skipped_per_class_good_reason, 1 )
          . '</b></td></tr>';

        $main_table .= '<tr><td class=l> . </td>';
        my @skipped_per_class_bad_reason;
        for (@classes) {
            $skipped = 0;
            for my $i (@skipped_lessons) {
                if ( $_ == $$i[0] and $$i[1] == -1 ) { $skipped++; }
            }
            if   ($skipped) { $main_table .= qq[<td>$skipped</td>]; }
            else            { $main_table .= qq[<td>&nbsp;</td>]; }
            push @skipped_per_class_bad_reason, $skipped;
        }
        $main_table .=
            '<td><b>'
          . ( sum \@skipped_per_class_bad_reason, 1 )
          . '</b></td></tr>';

        $main_table .= '<tr><td class=l>   1 </td>';
        my @skipped_lessons_avg_on_pupil;
        for ( my $i = 0 ; $i < @classes ; $i++ ) {
            my $tmp =
              ( $pupils_on_today[$i] and $skipped_per_class[$i] )
              ? sprintf( "%.1f", $skipped_per_class[$i] / $pupils_on_today[$i] )
              : 0;
            if ( $tmp > 0 ) {
                $main_table .= "<td>$tmp</td>";
                push @skipped_lessons_avg_on_pupil, $tmp;
            }
            else {
                $main_table .= qq[<td>&nbsp;</td>];
            }
        }
        $main_table .=
            '<td><b>'
          . ( avg \@skipped_lessons_avg_on_pupil, 1 )
          . '</b></td></tr>';
    }

    $main_table .= '<tr><td class=l></td>';
    my @dynamic;

    my $gdate_prev;
    if (   $periods_to_str_hash{$period} eq 't1'
        or $periods_to_str_hash{$period} eq 'q1'
        or $periods_to_str_hash{$period} eq 't'
        or $periods_to_str_hash{$period} eq 'e' )
    {
        $gdate_prev = $gdate;
        $gdate =~ s/(\d\d\d\d)$/$1-1/e;
    }
    for ( my $i = 0 ; $i < @classes ; $i++ ) {
        report( $classes[$i], $prev_period{ $periods_to_str_hash{$period} },
            \@out );
         if (   $out[7] == 0
            and $out[8] == 0
            and $out[11] == 0
            and $out[10] == 0
            and $out[6] == 0 )
        {
            $main_table .= qq[<td>&nbsp;</td>];
            push @dynamic, 0;
            next;
        }
        if ( @out and $out[8] ) {
            my $tmp = sprintf( "%.1f", $qua[$i] - $out[8] );
            push @dynamic, $tmp;
            $tmp =~ s/-/&minus;/;
            $main_table .= qq[<td>$tmp</td>];
        }
        else {
            push @dynamic, 0;
            if (@out) {
                $main_table .= qq[<td>0</td>];
            }
            else {
                $main_table .= qq[<td>&mdash;</td>];
            }
        }
    }

    {
        my $tmp = avg \@dynamic, 1;
        $tmp =~ s/-/&minus;/;
        $main_table .= '<td><b>' . $tmp . '</b></td></tr>';
    }

    if (   $periods_to_str_hash{$period} eq 't1'
        or $periods_to_str_hash{$period} eq 'q1'
        or $periods_to_str_hash{$period} eq 't'
        or $periods_to_str_hash{$period} eq 'e' )
    {
        $gdate = $gdate_prev;
    }

    my %accumulator_for_averange_rating;

    $main_table .= '<tr><td class=l> </td>';
    my %uou_rating;
    my $rating_value = 0;
    my @uou_rating;

    for ( reverse sort @qua ) {
        unless ( $uou_rating{$_} ) {
            $uou_rating{$_} = ++$rating_value;
        }
    }
    for ( my $i = 0 ; $i < @qua ; $i++ ) {
        $main_table .= qq[<td>$uou_rating{$qua[$i]}</td>];
        push @{ $accumulator_for_averange_rating{ $classes[$i] } },
          $uou_rating{ $qua[$i] };
    }
    $main_table .= '<td>&nbsp;</td></tr>';

    $main_table .= '<tr><td class=l> </td>';
    my %dynamic_rating;
    $rating_value = 0;
    for ( reverse sort { $a <=> $b } @dynamic ) {
        if ( $_ and !$dynamic_rating{$_} ) {
            $dynamic_rating{$_} = ++$rating_value;
        }
    }

    for ( my $i = 0 ; $i < @classes ; $i++ ) {
        if ( $dynamic_rating{ $dynamic[$i] } ) {
            $main_table .= qq[<td>$dynamic_rating{$dynamic[$i]}</td>];
            push @{ $accumulator_for_averange_rating{ $classes[$i] } },
              $dynamic_rating{ $dynamic[$i] };
        }
        else {
            $main_table .= '<td>&nbsp;</td>';
        }
    }
    $main_table .= '<td>&nbsp;</td></tr>';

    $main_table .= '<tr><td class=l> </td>';
    for ( my $i = 0 ; $i < @classes ; $i++ ) {
        my $tmp = avg( $accumulator_for_averange_rating{ $classes[$i] }, 1 );
        $main_table .= qq[<td>$tmp</td>];
    }
    $main_table .= '<td>&nbsp;</td></tr>';

    my $add_qua = '';

    if ( param 'add_qua' ) {
        my $add_qua_parallels = param 'add_qua_parallels';

        stop "      ."
          unless $add_qua_parallels;

        $add_qua = q[<br><table class="d pad" align=center cellspacing=0>
<tr>
  <th></th>
  <th>% </th>
</tr>];

        $add_qua_parallels =~ s/^\s*//;
        $add_qua_parallels =~ s/\s*$//;

        $add_qua_parallels =~ s/[\r\n]+/\n/g;

        my @add_qua_parallels_ranges = split( /\n/, $add_qua_parallels );

        s/\s*//g foreach @add_qua_parallels_ranges;

        my @qua_parallels_ranges;

        for (@add_qua_parallels_ranges) {
            my ( $from, $to ) = split /-/;
            sec_gt0 $from;
            sec_gt0 $to;
            if ( $to < $from ) {
                stop
"     ,  .<br>
   : 1-1, 1-2, 1-5; : 5-3, 10-1.<br><br>
 ,     :<br>
$from-$to.";
            }
            push @qua_parallels_ranges, [ $from, $to ];
        }

        my @range_qua;
        for (@qua_parallels_ranges) {
            $add_qua .= qq[<tr><td>$$_[0]&ndash;$$_[1]</td>];
            @range_qua = ();
            for ( my $i = 0 ; $i < @classes ; $i++ ) {
                if ( $class_hash{ $classes[$i] } =~ /^(\d+)/ ) {
                    if ( $1 >= $$_[0] and $1 <= $$_[1] ) {
                        push @range_qua, $qua[$i] if $qua[$i];
                    }
                }
            }
            $add_qua .= '<td>' . ( avg \@range_qua, 1 ) . '</td>';
            $add_qua .= q[</tr>];
        }

        $add_qua .= q[</table>];
    }

    my $add_uou = '';

    if ( param 'add_uou' ) {
        my $add_uou_parallels = param 'add_uou_parallels';
        stop "      ."
          unless $add_uou_parallels;
        $add_uou = q[<br><table class="d pad" align=center cellspacing=0>
<tr>
  <th></th>
  <th>% </th>
</tr>];

        $add_uou_parallels =~ s/^\s*//;
        $add_uou_parallels =~ s/\s*$//;
        $add_uou_parallels =~ s/[\r\n]+/\n/g;
        my @add_uou_parallels_ranges = split( /\n/, $add_uou_parallels );
        s/\s*//g foreach @add_uou_parallels_ranges;
        my @uou_parallels_ranges;
        for (@add_uou_parallels_ranges) {
            my ( $from, $to ) = split /-/;
            sec_gt0 $from;
            sec_gt0 $to;
            if ( $to < $from ) {
                stop
"     ,  .<br>
   : 1-1, 1-2, 1-5; : 5-3, 10-1.<br><br>
 ,     :<br>
$from-$to.";
            }
            push @uou_parallels_ranges, [ $from, $to ];
        }

        my @range_uou;
        for (@uou_parallels_ranges) {
            $add_uou .= qq[<tr><td>$$_[0]&ndash;$$_[1]</td>];
            @range_uou = ();
            for ( my $i = 0 ; $i < @classes ; $i++ ) {
                if ( $class_hash{ $classes[$i] } =~ /^(\d+)/ ) {
                    if ( $1 >= $$_[0] and $1 <= $$_[1] ) {
                        push @range_uou, $uou[$i] if $uou[$i];
                    }
                }
            }
            $add_uou .= '<td>' . ( avg \@range_uou, 1 ) . '</td>';
            $add_uou .= q[</tr>];
        }

        $add_uou .= q[</table>];
    }


    my @years = year_academic();

    tpl './tpl/report/progress_movement.html',
      {
        menu    => generate_menu(),
        footers => generate_footers(),
        period  => ' : '
          . $g_period_id_to_name{$period}
          . " ($years[0]&ndash;$years[1])",
        head_classes => $head_classes,
        main_table   => $main_table,
        add_qua      => $add_qua,
        add_uou      => $add_uou,
      };

}

sub report_totals_form() {
    tpl './tpl/report/totals_form.html',
      {
        menu         => generate_menu(),
        footers      => generate_footers(),
        filter_class => filter_class(0),
      };
}

sub report_totals() {
    my @c = param('c');
    unless ( scalar @c ) { stop "     ." }
    for (@c) { sec_gt0 $_; }

    my $exclude_empty_lines = param('exclude_empty_lines');

    my $page_break_threshold = param('page_break_threshold') || 210;
    sec_range( $page_break_threshold, 100, 900 );

    my %class_hash;
    fill_class_ref( \%class_hash );

    @c = sort { $class_hash{$a} cmp $class_hash{$b} } @c;

    my $periods_mode =
      sqlff 'SELECT F_INT FROM SP_SETTINGS WHERE NAME = \'periods_mode\'';
    my $head_periods;
    my @sql_fields;
    if ($periods_mode) {
        for ( 1 .. 4 ) { $head_periods .= qq[<th>$_<br>.</th>]; }
        @sql_fields = qw/q1 q2 q3 q4/;
    }
    else {
        for ( 1 .. 3 ) { $head_periods .= qq[<th>$_<br>.</th>]; }
        @sql_fields = qw/t1 t2 t3/;
    }
    my $sql = join( ',', @sql_fields );

    my @year = year_academic();

    my $t = '';

    my $counter;

    my $mm;
    my $mm_all = 0;

    my @subjects;
    sqla \@subjects, "select lesson_id, name from lesson order by name";

    my $table = '';

    my %scores;

    my $pupils_without_scores = '';

    for my $c (@c) {
        my @pupils;
        sqla \@pupils, "select human_id,last_name,first_name,middle_name
 from sys_matrix, human
where sys_matrix.class = $c and
      human_id = cast(vw_pupil as integer) and
      sys_matrix.view_class = 1 and
      exists(select 1 from get_node_saldo(sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)
order by last_name, first_name, middle_name";
        for my $pupil (@pupils) {
            $table =
qq[<table class="d" cellspacing=0><tr><th colspan="2">$$pupil[1] $$pupil[2] $$pupil[3], $class_hash{$c}<br>   $year[0]&ndash;$year[1] ..</th>$head_periods<th></th><th>.</th><th>.</th></tr>];
            $mm = 22.9;

            %scores = ();
            my $q = sqlp "select subj, y,e,e1,t,$sql from sp_scores_totals
           where uid = $$pupil[0]
           and scoreyear = $year[0]";
            my @a;
            while ( @a = $q->fetchrow_array ) {
                my $tmp = shift @a;
                $scores{"$tmp"} = [@a];
            }

            unless ( keys %scores ) {
                $pupils_without_scores .=
                  "$$pupil[1] $$pupil[2] $$pupil[3]<br>";
                next;
            }

            $counter = 1;
            for (@subjects) {
                my @tmp;

                if ($exclude_empty_lines) {
                    next unless $scores{ $$_[0] };
                    @tmp = @{ $scores{ $$_[0] } };
                }
                else {
                    @tmp = () unless $scores{$_};
                }

                $table .=
qq[<tr><td style="text-align:right">$counter</td><td style="text-align:left">$$_[1]</td>];
                $counter++;

                $tmp[0] ||= '&nbsp;';
                $tmp[1] ||= '&nbsp;';
                $tmp[3] ||= '&nbsp;';

                for ( my $i = 0 ; $i < scalar(@sql_fields) ; $i++ ) {
                    $tmp[ 4 + $i ] ||= '&nbsp;';
                    $table .= qq[<td>$tmp[4+$i]</td>];
                }

                $table .= qq[<td>$tmp[0]</td>];

                $table .= qq[<td>$tmp[1]];
                $table .= qq[/$tmp[2]] if $tmp[2];
                $table .= qq[</td>];

                $table .= qq[<td>$tmp[3]</td>];

                $table .= q[</tr>];
            }

            $table .= q[</table>];

            $mm += 3.3 * $counter - 1;

            my $class_master_text =
'  : __________&nbsp;&nbsp;&nbsp;&nbsp; : __________';

            $table .=
qq[<p style="border-bottom:1px dashed #ddd;font-size:8pt;margin:9px 0 10px 0;padding-bottom:10px;">$class_master_text</p>];

            if ( $mm_all + $mm >= $page_break_threshold ) {
                $table .=
q[<div style="page-break-after:always"><span style="display:none">&nbsp;</span></div>];
                $mm_all = $mm;
            }
            else {
                $mm_all += $mm;
            }

            $t .= $table;

        }
    }

    $pupils_without_scores =
      $pupils_without_scores
      ? "<br><br><b></b>: ,  ,   .<br>       .<br><br>"
      . $pupils_without_scores
      : '';

    tpl './tpl/report/totals.html',
      {
        menu                  => generate_menu(),
        footers               => generate_footers(),
        tables                => $t,
        pupils_without_scores => $pupils_without_scores,
      };
}

sub report_knowledge_area_form() {
    my %knowledge_area_ref;
    sqlh \%knowledge_area_ref,
      "select knowledge_id, name from knowledge_area_ref";

    my $options = '';
    for (
        sort { $knowledge_area_ref{$a} cmp $knowledge_area_ref{$b} }
        keys %knowledge_area_ref
      )
    {
        $options .= qq[<option value="$_">$knowledge_area_ref{$_}</option>];
    }

    my $periods_mode =
      sqlff "select F_INT from sp_settings where name = 'periods_mode'";
    my $periods;
    if ($periods_mode) {
        for ( 1 .. 4 ) {
            $periods .=
qq[<input type=radio id="idq$_" name="p" value="q$_"><label for="idq$_">$_ <br></label>];
        }
    }
    else {
        for ( 1 .. 3 ) {
            $periods .=
qq[<input type=radio id="idt$_" name="p" value="t$_"><label for="idt$_">$_ <br></label>];
        }
    }

    tpl './tpl/report/knowledge_area_form.html',
      {
        menu    => generate_menu(),
        footers => generate_footers(),
        options => $options,
        size    => scalar keys %knowledge_area_ref,
        classes => (
            $gteacher_show_all
            ? filter_class(0)
            : filter_class_teacher_binded(0)
        ),
        periods => $periods,
      };
}

sub report_knowledge_area {

    my $for_dynamic = shift || 0;
    my %previous;

    my %previous_qua;
    unless ($for_dynamic) {
        %previous_qua = report_knowledge_area(1);
    }

    my %knowledge_area_ref;
    sqlh \%knowledge_area_ref,
      "select knowledge_id, name from knowledge_area_ref";

    my @areas = param('a');
    unless (@areas) { stop "      ."; }
    sex_gt0(@areas);

    my @classes = param('c');
    unless (@classes) { stop "     ."; }
    sex_gt0(@classes);

    my $period = param('p');
    my $period_was;
    stop "   ." unless exists $g_period_to_str{$period};

    my $gdate_prev;
    if ($for_dynamic) {
        if (   $period eq 't1'
            or $period eq 'q1'
            or $period eq 't'
            or $period eq 'e' )
        {
            $gdate_prev = $gdate;
            $gdate =~ s/(\d\d\d\d)$/$1-1/e;
        }

        $period_was = $period;
        $period     = $prev_period{$period};
    }

    my %class_ref;
    fill_class_ref( \%class_ref );
    s/(\d+) /$1<br>/ for values %class_ref;
    my $classes_th = '';
    for (@classes) { $classes_th .= qq[<th>$class_ref{$_}</th>]; }
    my $class_commas = join ',', @classes;

    my @a;
    sqla \@a, "select lesson.lesson_id,
        lesson.name,
        knowledge_area_ref.knowledge_id
from lesson, knowledge_area_ref
where lesson.knowledge_area = knowledge_area_ref.knowledge_id
order by knowledge_area_ref.knowledge_id, lesson.name";

    my @tmp_pupils_in_classes;
    sqla \@tmp_pupils_in_classes, "select class, vw_pupil
from sys_matrix
where sys_matrix.view_class = 1
and class in ($class_commas)
and exists ( select 1 from get_node_saldo
(sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)";
    my %pupils_in_classes;
    for (@tmp_pupils_in_classes) {
        push @{ $pupils_in_classes{ $$_[0] } }, $$_[1];
    }
    @tmp_pupils_in_classes = undef;

    my @tmp_pupils_in_subgrp;
    sqla \@tmp_pupils_in_subgrp, "select id,uid from sp_class_subgrp
left join sp_class_subgrp_ref
on sp_class_subgrp.id=sp_class_subgrp_ref.id
where c in ($class_commas)";
    my %pupils_in_subgrp;
    for (@tmp_pupils_in_subgrp) {
        push @{ $pupils_in_subgrp{ $$_[0] } }, $$_[1];
    }
    @tmp_pupils_in_subgrp = undef;

    my $q = sqlp "select distinct vw_teacher, lesson, class, subgrp
from sys_matrix
where view_class = 9001
and class in ($class_commas)
and exists ( select 1 from get_node_saldo_by_unit ( sys_matrix.node_id, '$gdate', '', 0 )
where saldo_sum <> 0)";
    my @tmp;
    my %bind;
    while ( @tmp = $q->fetchrow_array ) {

        if ( $tmp[3] ) {
            push @{ $bind{"$tmp[1]"}{ $tmp[0] } }, "$tmp[2]|$tmp[3]";
        }
        else {
            push @{ $bind{"$tmp[1]"}{ $tmp[0] } }, $tmp[2];
        }

    }

    my ( $y, undef ) = year_academic();
    my @tmp_scores;
    sqla \@tmp_scores, "select uid||'|'||subj, $period
from sp_scores_totals
where scoreyear = $y";

    my %scores = map { $_->[0] => $_->[1] } @tmp_scores;
    @tmp_scores = undef;

    my %for_compare;
    my @dynamic;

    my $warnings = '';
    my @empty_area_ids;

    my $t = '';
    for (@areas) {
        $t .=
qq[<h2>  (%)    &laquo;$knowledge_area_ref{$_}&raquo;</h2>];
        $t .= qq[
<table class="d pad" align=center cellspacing=0>
<tr>
	<th></th><th></th>$classes_th<th></th><th></th>
</tr>];

        my %avg_for_column;

        my @total_avg_from_avg;

        @dynamic = ();

        my $line_count = 0;

        for my $x (@a) {

            if ( $$x[2] == $_ ) {

                if ( $bind{ $$x[0] } ) {

                    for my $bind_item (
                        sort { human_hash( $a, 1 ) cmp human_hash( $b, 1 ) }
                        keys %{ $bind{ $$x[0] } } )
                    {

                        $line_count++;

                        $t .= qq[<tr>];
                        $t .= qq[<td style="text-align:left">$$x[1]</td>];
                        $t .= qq[<td style="text-align:left" nowrap>]
                          . human_hash( $bind_item, 1 ) . '</td>';

                        my @sum_for_avg;
                        for my $c (@classes) {
                            if ( grep /^\Q$c\E$/,
                                @{ $bind{ $$x[0] }{$bind_item} } )
                            {
                                my ( $s5, $s4, $score_count ) = ( 0, 0, 0 );
                                for my $pupil ( @{ $pupils_in_classes{$c} } ) {
                                    $s5++ if $scores{"$pupil|$$x[0]"} == 5;
                                    $s4++ if $scores{"$pupil|$$x[0]"} == 4;
                                    $score_count++ if $scores{"$pupil|$$x[0]"};
                                }
                                if ($score_count) {
                                    my $qua =
                                      ( $s5 + $s4 ) * 100 / $score_count;

                                    if ( sprintf( "%.1f", $qua ) == 100 ) {
                                        $t .=
qq[<td><span style="color:#391">100</span>];
                                    }
                                    else {
                                        $t .= sprintf "<td>%.1f", $qua;
                                    }

                                    push @sum_for_avg, $qua;
                                    push @{ $avg_for_column{$c} }, $qua;
                                }
                                else {
                                    $t .=
qq[<td class=noscores style="text-align:center"> ];
                                }
                                $t .= qq[</td>];
                            }
                            elsif ( grep /$c\|/,
                                @{ $bind{ $$x[0] }{$bind_item} } )
                            {
                                my @tmp = grep /\|/,
                                  @{ $bind{ $$x[0] }{$bind_item} };
                                if ( $tmp[0] =~ m/^(\d+)\|(\d+)$/ ) {
                                    my ( $s5, $s4, $score_count ) = ( 0, 0, 0 );
                                    for my $pupil ( @{ $pupils_in_subgrp{$2} } )
                                    {
                                        $s5++ if $scores{"$pupil|$$x[0]"} == 5;
                                        $s4++ if $scores{"$pupil|$$x[0]"} == 4;
                                        $score_count++
                                          if $scores{"$pupil|$$x[0]"};
                                    }

                                    if ($score_count) {
                                        my $qua =
                                          ( $s5 + $s4 ) * 100 / $score_count;

                                        $t .= '<td>' . ( sprintf "%.1f", $qua );

                                        push @sum_for_avg, $qua;
                                        push @{ $avg_for_column{$c} }, $qua;
                                    }
                                    else {
                                        $t .=
qq[<td class=noscores style="text-align:center"> ];
                                    }

                                    $t .= qq[<br>:<br>];
                                    $t .= sqlff
"select name from sp_class_subgrp_ref where id = $2";
                                    $t .= qq[</td>];
                                }
                            }
                            else {
                                $t .= qq[<td>&nbsp;</td>];
                            }
                        }

                        if ($for_dynamic) {
                            if ( scalar @sum_for_avg ) {
                                $previous{"$bind_item|$$x[0]"} =
                                  avg( \@sum_for_avg, 1 );
                            }
                            else {
                                $previous{"$bind_item|$$x[0]"} = '-';
                            }
                        }

                        my $tmp_avg_avg = avg( \@sum_for_avg, 1 );
                        $t .= "<td>$tmp_avg_avg</td>";

                        if ( scalar(@sum_for_avg) ) {
                            push @total_avg_from_avg, $tmp_avg_avg;
                        }

                        if (    exists $previous_qua{"$bind_item|$$x[0]"}
                            and $previous_qua{"$bind_item|$$x[0]"} ne '-'
                            and scalar(@sum_for_avg) > 0 )
                        {
                            my $tmp = sprintf( "%.1f",
                                avg( \@sum_for_avg, 1 ) -
                                  $previous_qua{"$bind_item|$$x[0]"} );
                            push @dynamic, $tmp;

                            my $color;
                            if ( $tmp > 0 ) {
                                $color = "0,0,"
                                  . (
                                    10 * $tmp < 255 ? int( 10 * $tmp ) : 255 );
                            }
                            elsif ( $tmp < 0 ) {
                                $color =
                                  ( 10 * -$tmp < 255 ? int( 10 * -$tmp ) : 255 )
                                  . ",0,0";
                            }
                            else {
                                $color = "0,0,0";
                            }

                            $tmp =~ s/-/&minus;/;
                            $t .= qq[<td style="color:rgb($color)">$tmp</td>];
                        }
                        elsif ( exists $previous_qua{"$bind_item|$$x[0]"}
                            and $previous_qua{"$bind_item|$$x[0]"} eq '-' )
                        {
                            $t .= qq[<td>&mdash;</td>];
                        }
                        else {
                            $t .= qq[<td>&nbsp;</td>];
                        }

                        $t .= qq[</tr>];
                    }
                }
            }
        }

        unless ($line_count) {
            $warnings .= qq[<li>$knowledge_area_ref{$_}</li>];
            push @empty_area_ids, $_;
        }

        my $colspan = 2 + scalar @classes;

        $t .= q[<tr>];
        $t .=
qq[<td style="padding:3px 0; text-align:center" colspan=$colspan>   </td>];
        $t .= '<td><b>' . avg( \@total_avg_from_avg, 1 ) . '</b></td>';

        my $tmp = avg( \@dynamic, 1 );
        $tmp =~ s/-/&minus;/;
        $t .= "<td><b>$tmp</b></td></tr>";

        $t .= q[<tr>];
        $t .=
q[<td style="text-align:center;height:22px" colspan=2> <br> </td>];
        my @big_total;
        for my $c (@classes) {
            if ( defined $avg_for_column{$c} ) {
                my $avg = avg( $avg_for_column{$c} );
                push @big_total, $avg;
                $avg = sprintf( "%.1f", $avg );
                $for_compare{"$_|$c"} = $avg;

                if ( sprintf( "%.1f", $avg ) == 100 ) {
                    $t .= qq[<td><b style="color:#391">100</b>];
                }
                else {
                    $t .= sprintf "<td><b>%.1f</b></td>", $avg;
                }
            }
            else {
                $t .= '<td>&nbsp;</td>';
            }
        }

        $t .=
q[<td style="text-align:center" colspan=2><br><span style="font-size:14pt">]
          . avg( \@big_total, 1 )
          . q[</span></td>];
        $t .= q[</tr>];

        $t .= q[</table><br>];
    }

    if ($for_dynamic) {
        if (   $period_was eq 't1'
            or $period_was eq 'q1'
            or $period_was eq 't'
            or $period_was eq 'e' )
        {
            $gdate = $gdate_prev;
        }

        return %previous;
    }

    if ( scalar @areas > 1 ) {
        $t .= qq[<h2></h2>
<table class="d pad" cellspacing=0 align=center>
<tr><th></th>$classes_th</tr>];

        my %compare_sums;

        for my $a (@areas) {
            $t .=
              qq[<tr><td style="text-align:left">$knowledge_area_ref{$a}</td>];
            for (@classes) {
                if ( $for_compare{"$a|$_"} ) {
                    $t .= qq[<td>$for_compare{"$a|$_"}</td>];
                    push @{ $compare_sums{$_} }, $for_compare{"$a|$_"};
                }
                else {
                    $t .= "<td>&nbsp;</td>";
                }
            }
            $t .= '</tr>';
        }

        $t .= q[<tr><td style="padding:3px; text-align:center"></td>];
        for (@classes) {
            if ( scalar $compare_sums{$_} ) {
                $t .= qq[<td><b>] . avg( $compare_sums{$_}, 1 ) . q[</b></td>];
            }
            else {
                $t .= qq[<td>&nbsp;</td>];
            }
        }
        $t .= '</tr>';

        $t .= "</table>";
    }

    my @years = year_academic();

    if ( $warnings ne '' ) {
        my $classes;
        for (@classes) { $classes .= "&c=$_"; }

        my $all_areas_are_empty =
          ( scalar(@areas) == scalar(@empty_area_ids) )
          ? "(    ,   )<br><br>"
          : '';

        my $areas;
        for my $a (@areas) {
            unless ( grep /^\Q$a\E$/, @empty_area_ids ) {
                $areas .= "&a=$a";
            }
        }

        my $try_exclude_empty_area =
          $all_areas_are_empty
          ? ''
          : qq[<li> <a href="?act=report_knowledge_area&p=$period$classes$areas">       </a></li>];

        $warnings =
qq[<div align=center><div class="infoblock dontprint" style="width:650px">
<b>!<br> ,     ,
<br>       :<br></b>
<ul>$warnings</ul>$all_areas_are_empty
<b> ?</b>
<ul>
<li> ,     <a href="?act=teacher_placement"> </a>      </li>
<li>           </li>
$try_exclude_empty_area
</ul>
</div></div>];
    }

    tpl './tpl/report/knowledge_area.html',
      {
        menu    => generate_menu(),
        footers => generate_footers(),
        period  => ' : '
          . $g_period_to_name{$period}
          . " ($years[0]&ndash;$years[1])",
        warnings => $warnings,
        t        => $t,
      };
}

sub teacher_subjects() {
    my %class_hash;
    fill_class_ref( \%class_hash );

    my %bind;
    my $q = sqlp "select distinct last_name||' '||first_name||' '||middle_name,
(select name from lesson where lesson_id = sys_matrix.lesson),
sys_matrix.class
from sys_matrix join human on human_id = cast(sys_matrix.vw_teacher as integer)
where view_class = 9001
and exists ( select 1 from get_node_saldo_by_unit ( sys_matrix.node_id, '$gdate', '', 0 )
where saldo_sum <> 0)";
    my @a;
    while ( @a = $q->fetchrow_array ) {
        $class_hash{ $a[2] } ||= "<b style='color:red'>ID = $a[2]</b>";
        push @{ $bind{ $a[0] } }, "$a[1] &mdash; $class_hash{$a[2]}";
    }

    my $t;
    for ( sort keys %bind ) {
        $t .= qq[<b>$_</b><ol>];
        for my $l ( @{ $bind{$_} } ) {
            $t .= qq[<li>$l];
        }
        $t .= qq[</ol>];
    }

    tpl './tpl/teacher_subjects.html',
      {
        menu    => generate_menu(),
        footers => generate_footers(),
        t       => $t,
        date    => $gdate,
        ou      => MOUName('short'),
      };
}

sub req_view() {
    my ( $year_set, $year_set2 ) = year_academic();

    my ( $q, $s, $c );
    if ( param 'c' ) { sec_gt0 param 'c'; $c = param 'c' }
    if ( param 's' ) { sec_gt0 param 's'; $s = param 's' }

    my $filter_subj;
    my $filter_class;
    if ( $ggid == 2 ) {
        $filter_subj =
          $gteacher_show_all ? filter_subj($s) : filter_subj_teacher_binded($s);
        $filter_class =
          $gteacher_show_all
          ? filter_class($c)
          : filter_class_teacher_binded($c);
    }
    else {
        $filter_subj  = filter_subj($s);
        $filter_class = filter_class($c);
    }

    if ( $ggid == 2 ) {
        my $corrected_class = $c;
        my $corrected_subj  = $s;
        my $redir           = 0;
        unless ( $filter_class =~ m/value="$c"/ ) {
            $redir = 1;
            $filter_class =~ /value="(\d+)"/;
            $corrected_class = $1;
        }
        unless ( $filter_subj =~ m/value="$s"/ ) {
            $redir = 1;
            $filter_subj =~ /value="(\d+)"/;
            $corrected_subj = $1;
        }
        if ($redir) {
            redir "req_view&c=$corrected_class&s=$corrected_subj";
        }
    }

    my $q_cond;
    my @a;
    my @grp;
    my $grp_f;
    if ( param('c') =~ /^\d+$/ or param('s') =~ /^\d+$/ ) {
        if ( param 'c' and !param 's' ) { $q_cond = 'where c = ' . param('c'); }
        if ( param 's' and !param 'c' ) { $q_cond = 'where s = ' . param('s'); }
        if ( param 'c' and param 's' ) {
            $q_cond = 'where s = ' . param('s') . ' and c = ' . param('c');
        }
        $q = sqlp("select c, s, id, name from sp_class_subgrp_ref $q_cond");
        while ( @a = $q->fetchrow_array ) { push @grp, [@a] }
    }

    my %grp_req;
    $q = sqlp
"select c,s,g,f from sp_req where y = $year_set and g is not null and y = $year_set";
    while ( @a = $q->fetchrow_array ) { $grp_req{"c$a[0]s$a[1]g$a[2]"} = $a[3] }

    if ( param 'c' and !param 's' ) {
        $q = sqlp "select
paralel || (select name from class where class_id = sys_matrix.class),
name, lesson_id, class,
(select f from sp_req where y = $year_set and c = sys_matrix.class and s = lesson.lesson_id and g is null)
from sys_matrix, lesson
where
sys_matrix.view_class = 7
and sys_matrix.class = $c
and exists ( select 1 from get_node_saldo
  (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)
order by name";
    }
    elsif ( param 's' and !param 'c' ) {
        $q = sqlp "select
paralel || (select name from class where class_id = sys_matrix.class),
(select name from lesson where lesson_id = $s),
(select lesson_id from lesson where lesson_id = $s),
class,
(select f from sp_req, lesson where y = $year_set and c = sys_matrix.class and s = lesson.lesson_id and s = $s and g is null)
from sys_matrix
where
sys_matrix.view_class = 7
and exists ( select 1 from get_node_saldo
  (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0) order by paralel";
    }
    elsif ( param 'c' and param 's' ) {
        $q = sqlp "select
paralel || (select name from class where class_id = sys_matrix.class),
name, lesson_id, class,
(select f from sp_req where y = $year_set and c = sys_matrix.class and s = lesson.lesson_id and g is null)
from sys_matrix, lesson
where
sys_matrix.view_class = 7
and sys_matrix.class = $c
and lesson_id = $s
and exists ( select 1 from get_node_saldo
  (sys_matrix.node_id, '$gdate', 0) where saldo_sum <> 0)";
    }

    my $set_default = '';
    if ( isadmin() ) {
        $set_default =
          q[&nbsp;<a href="?act=set_default_sou_form"></a>];
    }

    if ( !param 'c' and !param 's' ) {
        my %replace = (
            menu        => generate_menu(),
            y1          => $year_set,
            y2          => $year_set2,
            table       => '',
            set_all     => '',
            set_default => $set_default,
            default_sou => '<b>'
              . (
                sqlff "select f_int from sp_settings where name = 'default_sou'"
              )
              . '</b>',
            filter_class => filter_class( param 'c' ),
            filter_subj  => $filter_subj,
            footers => '<div align=center><h2>   </h2>'
              . '   ,          .<br>'
              . '   ,        .</div>'
              . generate_footers(),
        );
        tpl( './tpl/setup_sou.html', \%replace );
    }

    my ( $t, $hiddens, $edit, $skip_this );
    while ( @a = $q->fetchrow_array() ) {
        $a[0] =~ s/([0-9]+.+\")/<b>$1<\/b>/;
        $a[4] = '&nbsp;' if !$a[4];
        $edit =
          $ggid == 0
          ? qq[<button type=button class="small req" onclick="setfactor('c$a[3]s$a[2]',1);" name=1>1</button><button type=button class="small req" onclick="setfactor('c$a[3]s$a[2]',2);" name=2>2</button><button type=button class="small req reqlast" onclick="setfactor('c$a[3]s$a[2]',3);" name=3>3</button>]
          : '';
        $skip_this = 0;
        for (@grp) {
            if ( $_->[0] == $a[3] and $_->[1] == $a[2] ) {
                $skip_this = 1;
                last;
            }
        }
        if ($skip_this) {
            for (@grp) {
                if ( $_->[0] == $a[3] ) {
                    $edit =
                      $ggid == 0
                      ? qq[<button type=button class="small req" onclick="setfactor('c$a[3]s$a[2]g$_->[2]',1);" name=1>1</button><button type=button class="small req" onclick="setfactor('c$a[3]s$a[2]g$_->[2]',2);" name=2>2</button><button type=button class="small req reqlast" onclick="setfactor('c$a[3]s$a[2]g$_->[2]',3);" name=3>3</button>]
                      : '';
                    $grp_f = $grp_req{"c$a[3]s$a[2]g$_->[2]"} || '&nbsp;';
                    $hiddens .=
qq[<input type=hidden name="c$a[3]s$a[2]g$_->[2]" value="">];
                    $t .=
qq[<tr OnMouseOver='m_over(this);' OnMouseOut='m_out(this);' class="def">
<td class='def subgrp' align=left nowrap>$a[0]<br><span class=subgrp>$_->[3]</span></td>
<td class='def' align=left nowrap>$a[1]</td>
<td id="td_c$a[3]s$a[2]g$_->[2]" class=def width=20 align=center><span style="font-weight: bold;" id="span_c$a[3]s$a[2]g$_->[2]">$grp_f</span></td>
<td class="def" style="width: auto;">
$edit
</td></tr>];
                }
            }
            next;
        }
        $hiddens .= qq[<input type=hidden name="c$a[3]s$a[2]" value="">];
        $t .=
qq[<tr OnMouseOver='m_over(this);' OnMouseOut='m_out(this);' class=def>
<td class='def pad_lr5' align=left nowrap>$a[0]</td>
<td class='def pad_lr5' align=left nowrap>$a[1]</td>
<td id="td_c$a[3]s$a[2]" class=def width=20 align=center><span style="font-weight: bold;" id="span_c$a[3]s$a[2]">$a[4]</span></td>
<td class="def pad_lr5" style="width: auto;">
$edit
</td></tr>];
    }

    my $set_all = '';
    if ( isadmin() ) {
        $set_all = q[&nbsp;&nbsp;|&nbsp;&nbsp; :
<a href="javascript:setfactorall('1');">1</a>
<a href="javascript:setfactorall('2');">2</a>
<a href="javascript:setfactorall('3');">3</a>];
    }

    my %replace = (
        menu        => generate_menu(),
        y1          => $year_set,
        y2          => $year_set2,
        colspan     => ( $ggid == 0 ) ? 2 : 0,
        set_all     => $set_all,
        set_default => $set_default,
        default_sou => '<b>'
          . ( sqlff "select f_int from sp_settings where name = 'default_sou'" )
          . '</b>',
        table        => $hiddens . $t,
        filter_class => $filter_class,
        filter_subj  => $filter_subj,
        footers      => generate_footers(),
    );
    tpl( './tpl/setup_sou.html', \%replace );
}

sub req_edit() {
    my ( $year_set, undef ) = year_academic();

    my ( $c, $s, $f, $g );
    foreach (param) {
        if ( m/^c(\d+)s(\d+)$/ and param $_) {
            ( $c, $s, $f ) = ( $1, $2, param $_);
            if ( sqlff
"select 1 from sp_req where y = $year_set and s = $s and c = $c and g is null"
              )
            { sql
"update sp_req set f = $f where y = $year_set and s = $s and c = $c and g is null";
            }
            else { sql
"insert into sp_req (y,c,s,f,g) values ($year_set,$c,$s,$f,NULL)";
            }
        }
        if ( m/^c(\d+)s(\d+)g(\d+)$/ and param $_) {
            ( $c, $s, $g, $f ) = ( $1, $2, $3, param $_);
            if ( sqlff
"select 1 from sp_req where y = $year_set and s = $s and c = $c and g = $g"
              )
            { sql
"update sp_req set f = $f where y = $year_set and s = $s and c = $c and g = $g";
            }
            else { sql
"insert into sp_req (y,c,s,f,g) values ($year_set,$c,$s,$f,$g)";
            }
        }
    }

    if   ( $ENV{'HTTP_REFERER'} ) { exit print redirect $ENV{'HTTP_REFERER'}; }
    else                          { redir('req_view'); }
}

sub set_default_sou_form() {
    my $sou = sqlff qq{select f_int from sp_settings where name = 'default_sou'}
      || 3;
    my $sou_list;
    for my $counter ( 1 .. 3 ) {
        $sou_list .=
          ( $sou == $counter )
          ? qq[<option value="$counter" selected>$counter</option>]
          : qq[<option value="$counter">$counter</option>];
    }

    my %replace = (
        menu    => generate_menu(),
        sou     => $sou_list,
        footers => generate_footers()
    );
    tpl( './tpl/setup_default_sou.html', \%replace );
}

sub set_default_sou() {
    my $sou = param 'sou';
    sec_range $sou, 1, 3;
    sql "update sp_settings set f_int = $sou where name = 'default_sou'";
    redir "req_view";
}

sub DrawVerticalText($$$) {
    my $id   = shift;
    my $text = shift;

    my $where = shift;

    my $delete = 0;
    if ( $where < 0 ) {
        $delete = 1;
        $where  = -$where;
    }

    my $path = '/tmp/img_subj/';
    if ( $where == 1 ) { $path = '/tmp/img_subgrp/' }

    if   ( defined $id ) { sec_gt0 $id }
    else                 { warn "DrawVerticalText: id not specified"; return }

    if ($delete) {
        unlink $gconf{sppath} . $path . $id . '.png';
        return;
    }

    my $file = $gconf{sppath} . $path . $id . '.png';
    open F, ">$file" or error "    $file: $!";
    binmode F;

    my $tmp = new GD::Image( 1, 1 );
    my $black_tmp = $tmp->colorAllocate( 0, 0, 0 );

    my @a =
      GD::Image->stringFT( $black_tmp, "./tahoma.ttf", 9, 1.57, 0, 0,
        str2unicode($text) );

    my ( $w, $h ) = ( $a[0] - $a[4] - 1, $a[1] - $a[5] - 1 );
    my $offsetx = $a[0] - $a[4] - $a[2] - 1;

    my $img = new GD::Image( $w, $h );

    my $white = $img->colorAllocate( 0xff, 0xff, 0xff );
    my $black = $img->colorAllocate( 0,    0,    0 );

    $img->stringFT( $black, "./tahoma.ttf", 9, 1.57, $offsetx, $h,
        str2unicode($text) );

    print F $img->png;

    close F;

}

1;
